/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.anzeigeAktorTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeAktorTls implements ClientSenderInterface
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static final Debug m_debug = Debug.getLogger();
  /**
   * Konfigurationsobjekt vom Typ typ.anzeigeAktorTls.
   */
  SystemObject m_anzeigeAktorTls;
  /**
   * Konfigurationsobjekt vom Typ typ.anzeige das der Aktor umsetzt.
   */
  SystemObject m_anzeige;
  /**
   * Instanz der Klasse mit den Umsetzung-Informationen.
   */
  AnzeigeTypUmsetzungTls m_anzeigeTypUmsetzung;
  /**
   * DE's die den Aktor realisieren.
   */
  HashMap<String, SystemObject> m_des = new HashMap<String, SystemObject>();
  /**
   * Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  SystemObject m_eak;
  /**
   * Kennung, ob die Plausibilitt publiziert werden soll (Modell vorhanden).
   */
  boolean m_publikationPlausibilitaet = false;
  /**
   * WvzCodePlausibilitt fr alles DEs (DE-Objekt als Key).
   */
  HashMap<SystemObject, WvzCodePlausibilitaet> m_plausibilitaet = new HashMap<SystemObject, WvzCodePlausibilitaet>();
  /**
   * Attributgruppe fr Versand der WZG-Plausibilitt.
   */
  AttributeGroup m_atg;
  /**
   * Aspect fr Versand der WZG-Plausibilitt. 
   */
  Aspect m_asp;
  /**
   * DataDescription fr Versand der WZG-Plausibilitt.
   */
  DataDescription m_dataDescription;

  /**
   * Konstruktor mit bergabe des Konfigurationsobjektes.
   * 
   * @param anzeigeAktorTls Konfigurationsobjekt ber das der Aktor versorgt ist.
   */
  public AnzeigeAktorTls(SystemObject anzeigeAktorTls)
  {
    m_anzeigeAktorTls = anzeigeAktorTls;

    DataModel dataModel = m_anzeigeAktorTls.getDataModel();
    
    m_atg = dataModel.getAttributeGroup("atg.anzeigeAktorTlsWvzStellZustand");
    m_asp = dataModel.getAspect("asp.plausibilittsPrfungLogisch");
    
    if ((m_atg != null) && (m_asp != null))
    {
    	m_dataDescription = new DataDescription(m_atg, m_asp);

    	try
			{
				AnzeigenGlobalZwischenSchichtTls.getConnection().subscribeSender(this, m_anzeigeAktorTls, m_dataDescription, SenderRole.source());
				
				m_publikationPlausibilitaet = true;
			}
			catch (OneSubscriptionPerSendData e)
			{
				e.printStackTrace();
			}
    }
  }
  
  /**
   * Holen der Pid des AnzeigeAktorTls.
   * 
   * @return Pid des AnzeigeAktorTls.
   */
  public String getPid()
  {
    if (m_anzeigeAktorTls != null)
      return m_anzeigeAktorTls.getPid();
    else
      return null;
  }

  /**
   * Holen der Anzeige die der Aktor umsetzt.
   * 
   * @return Anzeige die der Aktor umsetzt.
   */
  public SystemObject getAnzeige()
  {
    return m_anzeige;
  }
  
  /**
   * Setzen der Anzeige die der Aktor umsetzt.
   * 
   * @param anzeige Anzeige die der Aktor umsetzt.
   */
  public void setAnzeige(SystemObject anzeige)
  {
    m_anzeige = anzeige;
  }
  
  /**
   * Holen der Instanz der Klasse mit den Umsetzung-Informationen.
   * 
   * @return Instanz der Klasse mit den Umsetzung-Iinformationen.
   */
  public AnzeigeTypUmsetzungTls getAnzeigeTypUmsetzung()
  {
    return m_anzeigeTypUmsetzung;
  }
  
  /**
   * Setzen der Instanz der Klasse mit den Umsetzung-Informationen.
   * 
   * @param anzeigeTypUmsetzung Instanz der Klasse mit den Umsetzung-Informationen.
   */
  public void setAnzeigeTypUmsetzung(AnzeigeTypUmsetzungTls anzeigeTypUmsetzung)
  {
    m_anzeigeTypUmsetzung = anzeigeTypUmsetzung;
  }
  
  /**
   * Holen der HashMap mit den Konfigurationsobjekten vom Typ typ.de die den Aktor realisieren.
   * 
   * @return HashMap mit den Konfigurationsobjekten vom Typ typ.de die den Aktor realisieren.
   */
  public HashMap<String, SystemObject> getDes()
  {
    return m_des;
  }
  
  /**
   * Setzen der HashMap mit den Konfigurationsobjekten vom Typ typ.de die den Aktor realisieren.
   * 
   * @param des HashMap mit den Konfigurationsobjekten vom Typ typ.de die den Aktor realisieren.
   */
  public void setDes(HashMap<String, SystemObject> des)
  {
    m_des = des;
  }

  /**
   * Holen des Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   * 
   * @return Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  public SystemObject getEak()
  {
    return m_eak;
  }

  /**
   * Setzen des Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   * 
   * @param eak Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  public void setEak(SystemObject eak)
  {
    m_eak = eak;
  }

  /**
   * Bearbeitung der WZG-Plausibilitt.
   * 
   * @param de DE.
   * @param code WZG-Code.
   * @param plausibel Plausibilittsinformation.
   * @param datenzeitpunkt Datenzeitpunkt.
   */
  public void bearbeiteWvzCodePlausibilitaet(SystemObject de, int code, boolean plausibel, long datenzeitpunkt)
  {
  	System.out.println("AnzeigeAktorTls.bearbeiteWvzCodePlausibilitaet() " + de.getPid() + " Code " + code + " Plausibel " + plausibel);
    
  	WvzCodePlausibilitaet wvzCodePlausibilitaet = new WvzCodePlausibilitaet(de, code, plausibel);
  	
  	boolean versenden = false;
  	
  	WvzCodePlausibilitaet wvzCodePlausibilitaetAktuell = m_plausibilitaet.get(de);
  	
  	if (wvzCodePlausibilitaetAktuell == null)
  	{
  		versenden = true;
  	}
  	else
  	{
  		int codeAktuell = wvzCodePlausibilitaetAktuell.getCode();
  		
  		if (codeAktuell != code)
  		{
  			versenden = true;
  		}
  	}
  	
  	m_plausibilitaet.put(de, wvzCodePlausibilitaet);
  	
  	if ((versenden) && (m_publikationPlausibilitaet))
  	{
  		plausibilittPublizieren(datenzeitpunkt);
  	}
  	
  	if ((versenden) && (!plausibel))
  	{
  		MessageSender.getInstance().sendMessage("PlausibilittsfehlerLogischWzg-" + de.getPid(), MessageType.APPLICATION_DOMAIN, "PlausibilittsfehlerLogischWzg", MessageGrade.WARNING, de, MessageState.NEW_MESSAGE, new MessageCauser(null, "", ""), "WZG-Code " + code + " nicht plausibel");
  	}
  }

  /**
   * Publikation der AttributGruppe AnzeigeAktorTlsWvzStellZustand mit dem bergebenen Datenzeitpunkt.
   * 
   * @param datenzeitpunkt Datenzeitpunkt.
   */
  private void plausibilittPublizieren(long datenzeitpunkt)
	{
		Data data = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(m_atg);
		
		int anzahl = m_plausibilitaet.size();
		
		Set<SystemObject> keySet = m_plausibilitaet.keySet();
		
		Array array = data.getArray("WvzStellZustand");
		array.setLength(anzahl);
		
		int i = 0;
		
		for (SystemObject de : keySet)
		{
			WvzCodePlausibilitaet wvzCodePlausibilitaet = m_plausibilitaet.get(de);

			Data data2 = array.getItem(i);
			
			data2.getReferenceValue("DeWzgReferenz").setSystemObject(wvzCodePlausibilitaet.getDe());
			data2.getUnscaledValue("Code").set(wvzCodePlausibilitaet.getCode());
			data2.getScaledValue("CodePlausibel").setText(wvzCodePlausibilitaet.isPlausibel() ? "Ja" : "Nein");
			
			i++;
		}
		
		ResultData resultData = new ResultData(m_anzeigeAktorTls, m_dataDescription, datenzeitpunkt, data);
		
		try
		{
			AnzeigenGlobalZwischenSchichtTls.getConnection().sendData(resultData);
		}
		catch (DataNotSubscribedException e)
		{
			m_debug.error(e.getMessage());
			
			e.printStackTrace();
		}
		catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    String string = new String();

    string += "AnzeigeAktorTls";
    string += " pid " + getPid();

    if (m_anzeige != null)
    	string += ", Anzeige " + m_anzeige.getPid();

    if (m_anzeigeTypUmsetzung != null)
    	string += ", AnzeigeTypUmsetzung " + m_anzeigeTypUmsetzung.getPid();
    
    string += ", DEs: ";
    
    Set<String> liste = m_des.keySet();
    
    Iterator<String> iterator = liste.iterator();
    
    if (iterator.hasNext())
    {
      string += ((SystemObject)m_des.get(iterator.next())).getPid();
    }
    
    while(iterator.hasNext())
    {
      string += ", " + ((SystemObject)m_des.get(iterator.next())).getPid();
    }

    return string;
  }

	public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
	{
	}

	public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
	{
		return false;
	}
	
	class WvzCodePlausibilitaet
	{
		SystemObject m_de; 
		int m_code;
		boolean m_plausibel;
		
		public WvzCodePlausibilitaet(SystemObject de, int code, boolean plausibel)
		{
			m_de = de;
			m_code = code;
			m_plausibel = plausibel;
		}

		/**
		 * @return the de
		 */
		public SystemObject getDe()
		{
			return m_de;
		}

		/**
		 * @return the code
		 */
		public int getCode()
		{
			return m_code;
		}

		/**
		 * @return the plausibel
		 */
		public boolean isPlausibel()
		{
			return m_plausibel;
		}
	}
}
