/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.anzeigeQuerschnittAktorTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeQuerschnittAktorTls
{
  /**
   * Konfigurationsobjekt vom Typ typ.anzeigeQuerschnittAktorTls. 
   */
  SystemObject m_anzeigeQuerschnittAktorTls;
  /**
   * Konfigurationsobjekt vom Typ typ.anzeigeQuerschnitt das der Aktor umsetzt.
   */
  SystemObject m_anzeigeQuerschnitt;
  /**
   * Instanz der Klasse mit den Umsetzung-Informationen.
   */
  AnzeigeQuerschnittTypUmsetzungTls m_anzeigeQuerschnittTypUmsetzung;
  /**
   * DE die den Aktor realisieren.
   */
  SystemObject m_de;
  /**
   * Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  SystemObject m_eak;
  /**
   * Information, ob der AnzeigeQuerschnitt immer schaltbar ist, oder ein vorgelagerter Entscheider vorhanden ist.
   */
  boolean m_immerSchaltbar;
  
  /**
   * Konstruktor mit bergabe des Konfigurationsobjektes.
   * 
   * @param anzeigeQuerschnittAktorTls Konfigurationsobjekt ber das der Aktor versorgt ist.
   */
  public AnzeigeQuerschnittAktorTls(SystemObject anzeigeQuerschnittAktorTls)
  {
    m_anzeigeQuerschnittAktorTls = anzeigeQuerschnittAktorTls;
    
    m_immerSchaltbar = false;
  }
  
  /**
   * Holen der Pid des AnzeigeQuerschnittAktorTls.
   * 
   * @return Pid des AnzeigeQuerschnittAktorTls.
   */
  public String getPid()
  {
    if (m_anzeigeQuerschnittAktorTls != null)
      return m_anzeigeQuerschnittAktorTls.getPid();
    else
      return null;
  }

  /**
   * Holen des AnzeigeQuerschnitt den der Aktor umsetzt.
   * 
   * @return AnzeigeQuerschnitt den der Aktor umsetzt.
   */
  public SystemObject getAnzeigeQuerschnitt()
  {
    return m_anzeigeQuerschnitt;
  }
  
  /**
   * Setzen des AnzeigeQuerschnitt den der Aktor umsetzt.
   * 
   * @param anzeigeQuerschnitt AnzeigeQuerschnitt den der Aktor umsetzt.
   */
  public void setAnzeigeQuerschnitt(SystemObject anzeigeQuerschnitt)
  {
    m_anzeigeQuerschnitt = anzeigeQuerschnitt;
  }
  
  /**
   * Holen der Instanz der Klasse mit den Umsetzung-Informationen.
   * 
   * @return Instanz der Klasse mit den Umsetzung-Informationen.
   */
  public AnzeigeQuerschnittTypUmsetzungTls getAnzeigeQuerschnittTypUmsetzung()
  {
    return m_anzeigeQuerschnittTypUmsetzung;
  }
  
  /**
   * Setzen der Instanz der Klasse mit den Umsetzung-Informationen.
   * 
   * @param anzeigeQuerschnittTypUmsetzung Instanz der Klasse mit den Umsetzung-Informationen.
   */
  public void setAnzeigeQuerschnittTypUmsetzung(AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzung)
  {
    m_anzeigeQuerschnittTypUmsetzung = anzeigeQuerschnittTypUmsetzung;
  }
  
  /**
   * Holen des Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   * 
   * @return Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   */
  public SystemObject getDe()
  {
    return m_de;
  }
  
  /**
   * Setzen des Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   * 
   * @param de Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   */
  public void setDe(SystemObject de)
  {
    m_de = de;
  }

  /**
   * Holen des Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   * 
   * @return Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  public SystemObject getEak()
  {
    return m_eak;
  }

  /**
   * Setzen des Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   *  
   * @param eak Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  public void setEak(SystemObject eak)
  {
    m_eak = eak;
  }

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    String string = new String();

    string += "AnzeigeQuerschnittAktorTls";
    string += " pid " + getPid();

    if (m_anzeigeQuerschnitt != null)
    	string += ", AnzeigeQuerschnitt " + m_anzeigeQuerschnitt.getPid();

    if (m_anzeigeQuerschnittTypUmsetzung != null)
    	string += ", AnzeigeQuerschnittTypUmsetzung " + m_anzeigeQuerschnittTypUmsetzung.getPid();
    
    string += ", DE: " + m_de;
    
    return string;
  }

  /**
   * Holen der Information, ob der AnzeigeQuerschnitt immer schaltbar ist, oder ein vorgelagerter Entscheider vorhanden ist.
   * 
   * @return Information, ob der AnzeigeQuerschnitt immer schaltbar ist, oder ein vorgelagerter Entscheider vorhanden ist.
   */
  public boolean isImmerSchaltbar()
  {
    return m_immerSchaltbar;
  }

  /**
   * Setzen der Information, ob der AnzeigeQuerschnitt immer schaltbar ist, oder ein vorgelagerter Entscheider vorhanden ist.
   * 
   * @param immerSchaltbar Information, ob der AnzeigeQuerschnitt immer schaltbar ist, oder ein vorgelagerter Entscheider vorhanden ist.
   */
  public void setImmerSchaltbar(boolean immerSchaltbar)
  {
    m_immerSchaltbar = immerSchaltbar;
  }
}
