/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der AnzeigeQuerschnitt-Informationen fr NRW.
 * Diese Klasse wird ber die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung berschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeQuerschnittTypUmsetzungTlsNrw1 extends AnzeigeQuerschnittTypUmsetzungTlsDefault
{  
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeQuerschnittAktorTlsDes Liste mit AnzeigeQuerschnittAktorTls mit Key DEs.
   * @param listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt Liste mit AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   * @param listeAnzeigeQuerschnittEigenschaftIst Liste mit Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   * @param listeAnzeigeQuerschnittSchaltbarkeitNba Liste mit Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   * @param listeAnzeigeQuerschnittStatus Liste mit Status Methoden fr alle AnzeigenQuerschnitte
   */
  public AnzeigeQuerschnittTypUmsetzungTlsNrw1(HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsDes, HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt, HashMap<String, Data> listeAnzeigeQuerschnittEigenschaftIst, HashMap<String, Data> listeAnzeigeQuerschnittSchaltbarkeitNba, HashMap<String, HashMap<String, Integer>> listeAnzeigeQuerschnittStatus)
  {
    super(listeAnzeigeQuerschnittAktorTlsDes, listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt, listeAnzeigeQuerschnittEigenschaftIst, listeAnzeigeQuerschnittSchaltbarkeitNba, listeAnzeigeQuerschnittStatus);

    m_debug.finer("AnzeigeTypUmsetzungTlsNrw1");
  }
  
  /**
   * Abarbeitung der Erweiterung fr die NBA Kln-Koblenz in den Unterzentralen von Heusch-Boesefeldt.
   * Abarbeitung der Betriebsarts des vorgelagerten Entscheiders entsprechend den Vorgaben.
   * 
   * @param result Attributgruppe atg.tlsWzgBetriebsart mit Aspekt asp.tlsVorgabeBedienung.
   */
  public Data tlsWzgBetriebsArtNba(ResultData result)
  {
    SystemObject object = result.getObject();

    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

    if (anzeigeQuerschnittAktor == null)
      return null;
    
    String anzeigeQuerschnittPid = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt().getPid();
    
    Data dataAnzeigeQuerschnittSchaltbarkeitNba = (Data)getListeAnzeigeQuerschnittSchaltbarkeitNba().get(anzeigeQuerschnittPid);
    
    m_debug.finer("tlsWzgBetriebsart DE " + object.getPid() + "\nAnzeigeQuerschnitt " + anzeigeQuerschnittPid + "\nresult " + result + "\nData " + dataAnzeigeQuerschnittSchaltbarkeitNba);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      dataAnzeigeQuerschnittSchaltbarkeitNba.getScaledValue("DurchNbaSchaltbar").setText(durchNbaSchaltbarZustandBestimmen(anzeigeQuerschnittAktor.isImmerSchaltbar(), "nicht ermittelbar"));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
        
        String betriebsart = data.getScaledValue("BetriebsArt").getValueText();
        
        String betriebszustand = null;
        
        if (betriebsart.equals("Normalbetrieb"))
        {
          betriebszustand = "Ja";
        }
        
        if (betriebsart.equals("Handbetrieb"))
        {
          betriebszustand = "Nein";
        }
        
        if (betriebszustand == null)
        {
          betriebszustand = "nicht ermittelbar";
        }
        
        dataAnzeigeQuerschnittSchaltbarkeitNba.getScaledValue("DurchNbaSchaltbar").setText(durchNbaSchaltbarZustandBestimmen(anzeigeQuerschnittAktor.isImmerSchaltbar(), betriebszustand));
      }
      else // keine Daten vorhanden
      {
        dataAnzeigeQuerschnittSchaltbarkeitNba.getScaledValue("DurchNbaSchaltbar").setText(durchNbaSchaltbarZustandBestimmen(anzeigeQuerschnittAktor.isImmerSchaltbar(), "nicht ermittelbar"));
      }
    }
    
    return dataAnzeigeQuerschnittSchaltbarkeitNba;
  }
}
