/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import de.bsvrz.dav.daf.main.Data;

/**
 * Statische Klasse fr die Bestimmung der Befehlsart aus der Attributgruppe atg.anzeigeEigenschaftSoll 
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeSollBefehlsArt
{
  /**
   * Befehlsart ist undefiniert.
   */
  static final int Undefiniert = 0;
  /**
   * Befehlsart WVZ-Inhalt.
   */
  static final int WvzInhalt = 1;
  /**
   * Befehlsart Grafik.
   */
  static final int Grafik = 2;
  /**
   * Befehlsart Wechseltext.
   */
  static final int Wechseltext = 3;
  
  /**
   * Leerer Konstruktor.
   */
  public AnzeigeSollBefehlsArt()
  {
    
  }
  
  /**
   * Bestimmung der Befehlsart fr eine Anzeige aus dem Inhalt der AnzeigeEigenschaftSoll-AttributGruppe.
   * 
   * @param data Daten der AnzeigeEigenschaftSoll-AttributGruppe.
   * @return Befehlsart entsprechend der definierten Befehlsarten dieser Klasse.
   */
  static private int befehlsArtBestimmen(Data data)
  {
    if (data.getReferenceValue("AnzeigeInhalt").getSystemObject() != null)
    {
      return WvzInhalt; 
    }
    
    if (data.getReferenceArray("GrafikDarstellungen").getLength() != 0)
    {
      return Grafik; 
    }
    
    if (data.getReferenceValue("Wechseltext").getSystemObject() != null)
    {
      return Wechseltext; 
    }
    
    if (data.getItem("DynamischerWechseltext").getReferenceValue("Font").getSystemObject() != null)
    {
      return Wechseltext; 
    }
    
    return Undefiniert;
  }
  
  /**
   * Bestimmung der Befehlsart fr eine Anzeige aus dem Inhalt der AnzeigeEigenschaftSoll-AttributGruppe.
   * 
   * @param data Daten der AnzeigeEigenschaftSoll-AttributGruppe.
   * @return Befehlsart entsprechend der definierten Befehlsarten dieser Klasse.
   */
  static public int eigenschaftBefehlsArtBestimmen(Data data)
  {
    return befehlsArtBestimmen(data.getItem("Eigenschaft"));	
  }

  /**
   * Bestimmung der Befehlsart fr eine Anzeige aus dem Inhalt der AnzeigeAutarkSoll-AttributGruppe.
   * 
   * @param data Daten der AnzeigeAutarkSoll-AttributGruppe.
   * @return Befehlsart entsprechend der definierten Befehlsarten dieser Klasse.
   */
  static public int autarkBefehlsArtBestimmen(Data data)
  {
    return befehlsArtBestimmen(data.getItem("Autark"));	
  }
}
