/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der Anzeige-Informationen fr NW LEH Kln-Lvenich.
 * Diese Klasse wird ber die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung berschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2014/02/13 16:28:12 $ / ($Author: Drapp $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsNwLeh1 extends AnzeigeTypUmsetzungTlsDefault
{
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden fr alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsNwLeh1(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsNwLeh1(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }
}
