/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls.anzeigenglobal;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittAktorTls;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittQuellenVlt;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittTypUmsetzungTls;
import de.bsvrz.ste.agzs.tls.GlobalUmsetzung;
import de.bsvrz.ste.agzs.tls.AnzeigenGlobalZwischenSchichtTls;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.anzeigeQuerschnitt 
 * und die Abarbeitung der ber diesen Typ enpfangenen spezifischen Attributgruppen.
 * Die Abarbeitung erfolgt mit den versorgten Umsetzungsklassen, oder wenn keine spezielle versorgt, mit der Standardumsetzung.<br>
 * Die Standardumsetzungsklasse ist die Klasse 'zswzg.kex.tls.AnzeigeQuerschnittTypUmsetzungTlsDefault'.<br>
 * <br>
 * Abarbeitung: AnzeigenGlobal -> KExTls.<br>
 *
 * @version $Revision: 1.7 $ / $Date: 2012/11/29 11:49:43 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeQuerschnitt implements ClientReceiverInterface, ClientSenderInterface
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Konfigurationsobjekt vom Typ typ.anzeigeQuerschnitt.
   */
  private ConfigurationObject m_anzeigeQuerschnitt;
  /**
   * Datenverteilerverbindung.
   */
  private ClientDavInterface m_connection;
  /**
   * Instanz der Klasse ZwischenschichtAq.
   */
  private AnzeigenGlobalZwischenSchichtTls m_zwischenschichtAq;
  /**
   * Datenmodell.
   */
  private DataModel m_dataModel;
  /**
   * Instanz der Klasse Dirigent dem der AnzeigeQuerschnitt angehrt.
   */
  private Dirigent m_dirigent = null;
  /**
   * AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   */
  private HashMap<String, AnzeigeQuerschnittAktorTls> m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt;
  /**
   * Defaultklasse fr die AnzeigeQuerschnittTypUmsetzungTls.
   */
  private String klassenNameDefault = "zswzg.kex.tls.AnzeigeQuerschnittTypUmsetzungTlsDefault";
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung der AnzeigeQuerschnitt-Helligkeit.
   */
  private String[][] methodenHelligkeit =
  {
    {"atg.anzeigeQuerschnittEigenschaftSoll", "anzeigeQuerschnittEigenschaftSollHelligkeit"},
  };
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung der AnzeigeQuerschnitt-Betriebsart.
   */
  private String[][] methodenBetriebsart =
  {
    {"atg.anzeigeQuerschnittEigenschaftSoll", "anzeigeQuerschnittEigenschaftSollBetriebsart"},
  };
  /**
   * Methode fr die Umsetzung der AnzeigeQuerschnitt-Stromausfall.
   */
  private String methodeStromausfall = "anzeigeQuerschnittStromausfall";
  /**
   * Methode fr die Umsetzung der AnzeigeQuerschnitt-Trkontakt.
   */
  private String methodeTuerkontakt = "anzeigeQuerschnittTuerkontakt";
  
  /**
   * Liste mit Anzeigen, die dem AnzeigeQuerschnitt angehren.
   */
  private Vector<Anzeige> m_anzeigen = new Vector<Anzeige>();
  /**
   * AttributGruppe-Aspekt-Kombinationen fr den Datenversand.
   */
  private String[][] m_datenversandAttributGruppeAspekt = 
  {
    {"atg.anzeigeQuerschnittEigenschaftIst", "asp.aqZustand"},
    {"atg.anzeigeQuerschnittSchaltbarkeitNba", "asp.aqZustand"},
  };
  /**
   * Puffer fr Eingangsdaten Anzeige-Befehle bis zur Synchronisierung.
   */
  private Vector<ResultData> m_eingangsPufferAnzeigeBefehle = new Vector<ResultData>();
  /**
   * Puffer fr Eingangsdaten Autark-Befehle bis zur Synchronisierung.
   */
  private Vector<ResultData> m_eingangsPufferAutarkBefehle = new Vector<ResultData>();
  
  private HashMap<SystemObject, DataDescription> m_zusaetzlichAnzumeldendeDaten = new HashMap<SystemObject, DataDescription>();
  private AnzeigeQuerschnittQuellenVlt m_anzeigeQuerschnittQuellenVlt;

  /**
   * Konstruktor mit bergabe der notwendigen Parameter.
   * 
   * @param connection Datenverteilerverbindung.
   * @param zwischenschichtAq Instanz der Klasse ZwischenschichtAq.
   * @param anzeigeQuerschnitt Konfigurationsobjekt vom Typ typ.anzeigeQuerschnitt.
   * @param listeAnzeigeKlasse Liste mit Anzeigen, die dem AnzeigeQuerschnitt angehren.
   * @param listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   */
  public AnzeigeQuerschnitt(ClientDavInterface connection,
                            AnzeigenGlobalZwischenSchichtTls zwischenschichtAq,
                            ConfigurationObject anzeigeQuerschnitt, 
                            Vector<Anzeige> listeAnzeigeKlasse,
                            HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt)
  {
    m_connection = connection;
    m_zwischenschichtAq = zwischenschichtAq;
    m_anzeigeQuerschnitt = anzeigeQuerschnitt;
    m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt = listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt;
    
    m_dataModel = m_connection.getDataModel();

//    m_debug.error("AnzeigeQuerschnitt " + m_anzeigeQuerschnitt.getPid());
    
    List anzeigen = m_anzeigeQuerschnitt.getNonMutableSet("Anzeigen").getElements();
    
//    System.out.println("AnzeigeQuerschnitt " + m_anzeigeQuerschnitt.getPid());
    
    for (int i = 0; i < anzeigen.size(); i++)
    {
      String pidAnzeige = ((SystemObject)anzeigen.get(i)).getPid();

      for (int j = 0; j < listeAnzeigeKlasse.size(); j++)
      {
        Anzeige anzeigeTmp = listeAnzeigeKlasse.get(j);
        
        if (pidAnzeige.equals(anzeigeTmp.getPid()))
        {
          anzeigeTmp.setAnzeigeQuerschnitt(this);
          
          m_anzeigen.add(anzeigeTmp);
          
          break;
        }
      }
    }
    
    // Daten der AttributGruppe atg.anzeigeQuerschnittQuellenVlt einlesen, wenn vorhanden
    
    m_anzeigeQuerschnittQuellenVlt = new AnzeigeQuerschnittQuellenVlt(m_connection, m_anzeigeQuerschnitt);
    
    if (m_anzeigeQuerschnittQuellenVlt.isVersorgt())
    {
      m_debug.config("AtgAnzeigeQuerschnittQuellenVlt fr " + m_anzeigeQuerschnitt + " versorgt");
  
      if (m_anzeigeQuerschnittQuellenVlt.isVersorgtStromausfall())
      {
        m_debug.config("AtgAnzeigeQuerschnittQuellenVlt-Stromausfall fr " + m_anzeigeQuerschnitt + " versorgt");

        DataDescription dataDescription = new DataDescription(m_anzeigeQuerschnittQuellenVlt.getAtgStromausfall(), m_anzeigeQuerschnittQuellenVlt.getAspStromausfall());
        
        List<SystemObject> m_desStromausfall = m_anzeigeQuerschnittQuellenVlt.getDesStromausfall();
        
        for (SystemObject de : m_desStromausfall)
        {
          m_zusaetzlichAnzumeldendeDaten.put(de, dataDescription);
        }
      }
      
      if (m_anzeigeQuerschnittQuellenVlt.isVersorgtTuerkontakt())
      {
        m_debug.config("AtgAnzeigeQuerschnittQuellenVlt-Trkontakt fr " + m_anzeigeQuerschnitt + " versorgt");

        DataDescription dataDescription = new DataDescription(m_anzeigeQuerschnittQuellenVlt.getAtgTuerkontakt(), m_anzeigeQuerschnittQuellenVlt.getAspTuerkontakt());
        
        List<SystemObject> m_desTuerkontakt = m_anzeigeQuerschnittQuellenVlt.getDesTuerkontakt();
        
        for (SystemObject de : m_desTuerkontakt)
        {
          m_zusaetzlichAnzumeldendeDaten.put(de, dataDescription);
        }
      }
    }
  }

  /**
   * Anmelden fr den Datenversand.
   * 
   * @param sender Instanz des Senders.
   */
  protected void anmeldungDatenversand(ClientSenderInterface sender)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // AnzeigeQuerschnitt auf Datenversand anmelden

    for (int i = 0; i < m_datenversandAttributGruppeAspekt.length; i++)
    {
      atgDaten = m_dataModel.getAttributeGroup(m_datenversandAttributGruppeAspekt[i][0]);
      aspDaten = m_dataModel.getAspect(m_datenversandAttributGruppeAspekt[i][1]);
      ddDaten = new DataDescription(atgDaten, aspDaten);
      
      m_debug.config("Anmeldung Quelle " + m_anzeigeQuerschnitt + " " + ddDaten);
  
      try
      {
        m_connection.subscribeSender(sender, m_anzeigeQuerschnitt, ddDaten, SenderRole.source());
      }
      catch (OneSubscriptionPerSendData e)
      {
        e.printStackTrace();
  
        System.exit(0);
      }
    }
  }

  /**
   * Abmelden vom Datenversand.
   * 
   * @param sender Instanz des Senders.
   */
  protected void abmeldungDatenversand(ClientSenderInterface sender)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // AnzeigeQuerschnitt vom Datenversand abmelden
        
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst");
    aspDaten = m_dataModel.getAspect("asp.aqZustand");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Abmeldung " + m_anzeigeQuerschnitt + " " + ddDaten);

    m_connection.unsubscribeSender(sender, m_anzeigeQuerschnitt, ddDaten);
  }

  /**
   * Anmelden fr den Datenempfang.
   * 
   * @param empfaenger Instanz des Empfngers.
   */
  protected void anmeldungDatenempfang(ClientReceiverInterface empfaenger)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // AnzeigeQuerschnitt auf Datenempfang anmelden
    
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftSoll");
    aspDaten = m_dataModel.getAspect("asp.aqVorgabe");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Anmeldung Senke " + m_anzeigeQuerschnitt.getPid() + " " + ddDaten);

    try
    {
      m_connection.subscribeReceiver(empfaenger, m_anzeigeQuerschnitt, ddDaten, ReceiveOptions.normal(), ReceiverRole.drain());
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();

      System.exit(0);
    }
  }

  /**
   * Abmelden vom Datenempfang.
   * 
   * @param empfaenger Instanz des Empfngers.
   */
  protected void abmeldungDatenempfang(ClientReceiverInterface empfaenger)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // AnzeigeQuerschnitt vom Datenempfang abmelden
    
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftSoll");
    aspDaten = m_dataModel.getAspect("asp.aqVorgabe");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Abmeldung " + m_anzeigeQuerschnitt.getPid() + " " + ddDaten);

    m_connection.unsubscribeReceiver(empfaenger, m_anzeigeQuerschnitt, ddDaten);

    if (m_zusaetzlichAnzumeldendeDaten.size() > 0)
    {
      Set<SystemObject> keySet = m_zusaetzlichAnzumeldendeDaten.keySet();
      
      for (SystemObject key : keySet)
      {
        DataDescription dataDescription = m_zusaetzlichAnzumeldendeDaten.get(key);

        m_connection.unsubscribeReceiver(this, key, dataDescription);
      }
    }
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientReceiverInterface#update(stauma.dav.clientside.ResultData[])
   */
  public void update(ResultData[] results)
  {
    for (ResultData result : results)
    {
//      System.out.println("AnzeigeQuerschnitt Result " + result);
      
      String atg = result.getDataDescription().getAttributeGroup().getPid();
      
      if (atg.equals("atg.anzeigeQuerschnittEigenschaftSoll"))
      {
        if (result.hasData())
        {
          anzeigeQuerschnittEigenschaftenSollVerarbeiten(result);
        }
        else // Eingangspuffer lschen
        {
          getEingangsPufferAnzeigeBefehle().removeAllElements();
        }
      }
      else if (atg.equals("atg.anzeigeEigenschaftSoll"))
      {
        anzeigeEigenschaftSollVerarbeiten(result);
      }
      else if (atg.equals("atg.anzeigeAutarkSoll"))
      {
        anzeigeAutarkSollVerarbeiten(result);
      }
      else
      {
        if (m_anzeigeQuerschnittQuellenVlt.isVersorgtStromausfall())
        {
          if (atg.equals(m_anzeigeQuerschnittQuellenVlt.getAtgStromausfall().getPid()))
          {
            boolean stromausfall = false;
            
            if (result.hasData())
            {
              Data data = result.getData();
              
              String text = data.getScaledValue(m_anzeigeQuerschnittQuellenVlt.getAttributStromausfall()).getValueText();
              
              if (text.equals(m_anzeigeQuerschnittQuellenVlt.getAttributWertStromausfall()))
              {
                stromausfall = true;
              }
            }

            m_debug.fine("Stromausfall " + stromausfall + " am AnzeigeQuerschnitt " + m_anzeigeQuerschnitt);

            anzeigeQuerschnittStromausfallVerarbeiten(result, m_anzeigeQuerschnitt, result.getObject(), stromausfall);
          }
        }

        if (m_anzeigeQuerschnittQuellenVlt.isVersorgtTuerkontakt())
        {
          if (atg.equals(m_anzeigeQuerschnittQuellenVlt.getAtgTuerkontakt().getPid()))
          {
            boolean tuerkontakt = false;
            
            if (result.hasData())
            {
              Data data = result.getData();
              
              String text = data.getScaledValue(m_anzeigeQuerschnittQuellenVlt.getAttributTuerkontakt()).getValueText();
              
              if (text.equals(m_anzeigeQuerschnittQuellenVlt.getAttributWertTuerkontakt()))
              {
                tuerkontakt = true;
              }
            }
            
            m_debug.fine("Trkontakt " + tuerkontakt + " am AnzeigeQuerschnitt " + m_anzeigeQuerschnitt);

            anzeigeQuerschnittTuerkontaktVerarbeiten(result, m_anzeigeQuerschnitt, result.getObject(), tuerkontakt);
          }
        }
      }
    }
  }

  /**
   * Verarbeiten der Befehle fr den AnzeigeQuerschnitt.
   * 
   * @param resultData Befehle fr den AnzeigeQuerschnitt.
   */
  private void anzeigeQuerschnittEigenschaftenSollVerarbeiten(ResultData resultData)
  {
    ResultData ergebnis = null;

    boolean datenVerarbeitet = false;
    
    Vector<ResultData> atgSenden = new Vector<ResultData>();
    
//    System.out.println("Folgende Anzeigen bearbeiten:");
    
    Data data = resultData.getData();
    
    long zeitpunktUebergeordnet = data.getTimeValue("SollZeitpunkt").getMillis();
    
    ReferenceArray array = data.getReferenceArray("Anzeigen");
    
//    for (int i = 0; i < array.getLength(); i++)
//    {
//      SystemObject objekt = array.getReferenceValue(i).getSystemObject();
//
//      System.out.println("- Anzeige " + objekt.getPid());
//    }

    for (int i = 0; i < array.getLength(); i++)
    {
      SystemObject objekt = array.getReferenceValue(i).getSystemObject();

      for (int j = 0; j < getEingangsPufferAnzeigeBefehle().size(); j++)
      {
        ResultData resultDataSenden = getEingangsPufferAnzeigeBefehle().get(j);
        
        if (resultDataSenden.getObject().getPid().equals(objekt.getPid()))
        {
          for (int k = 0; k < m_anzeigen.size(); k++)
          {
            Anzeige anzeige = m_anzeigen.get(k);
            
            if (anzeige.getPid().equals(objekt.getPid()))
            {
              ResultData result1 = anzeige.anzeigeVerarbeiten(resultDataSenden, new Long(zeitpunktUebergeordnet));
              
              if (result1 != null)
              {
                atgSenden.add(result1);
              }
              
//              System.out.println("result1 " + result1);
            }
          }
        }
      }
    }
    
    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktorUms = (AnzeigeQuerschnittAktorTls)m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt.get(getPid());
    
    if (anzeigeQuerschnittAktorUms != null)
    {
      SystemObject eak = anzeigeQuerschnittAktorUms.getEak();

      Data dataEak = m_connection.createData(m_dataModel.getAttributeGroup("atg.tlsEinzelTelegrammSenden"));
      
      dataEak.getScaledValue("Senden").setText("senden");
      
      AttributeGroup atgEakErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsEinzelTelegrammSenden");
      Aspect aspEakErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect("asp.tlsVorgabe");
      
      ResultData resultEak = new ResultData(eak, new DataDescription(atgEakErgebnis, aspEakErgebnis), zeitpunktUebergeordnet, dataEak);

      atgSenden.add(resultEak);

//      System.out.println("resultEak " + resultEak);
    }
    
    Date zeitpunkt = new Date();
    
    for (int i = 0; i < atgSenden.size(); i++)
    {
      ResultData res = atgSenden.get(i);        
      
      res.setDataTime(zeitpunkt.getTime());
      
      try
      {
//        System.out.println("Senden " + atgSenden.get(i));
        
        m_connection.sendData(res);
      }
      catch (DataNotSubscribedException e)
      {
        DataDescription dd = res.getDataDescription();
        List<SystemObject> objekte = new ArrayList<SystemObject>();
        objekte.add(res.getObject());
        
        try
        {
          m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());

          m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
        }
        catch (OneSubscriptionPerSendData e2)
        {
          e2.printStackTrace();
        } 

        try
        {
          Thread.sleep(500);
        }
        catch (InterruptedException e2)
        {
          e2.printStackTrace();
        }
        
        try
        {
          m_connection.sendData(res);
        }
        catch (DataNotSubscribedException e1)
        {
          e1.printStackTrace();
        }
        catch (SendSubscriptionNotConfirmed e1)
        {
          m_debug.warning("Ich 3", e1);
//          e1.printStackTrace();
        }
      }
      catch (SendSubscriptionNotConfirmed e)
      {
        m_debug.warning("Ich 4", e);

//        e.printStackTrace();
      }
    }
    
    getEingangsPufferAnzeigeBefehle().removeAllElements();

    // Helligkeit umsetzen
    
    if (anzeigeQuerschnittAktorUms != null)
//    if ((anzeigeQuerschnittAktorUms != null) && (anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getListeUmsetzungHelligkeit().size() > 0))
    {
      m_debug.fine("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + resultData.toString());
//      m_debug.error("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + result.toString());

      SystemObject object = resultData.getObject();
      
      AttributeGroup atg = resultData.getDataDescription().getAttributeGroup();
      Aspect         asp = resultData.getDataDescription().getAspect();

      AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();
      
      int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungHelligkeit().size();

      for (int i = 0; i < anzahlGlobalUmsetzung; i++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungHelligkeit().get(i);
          
        if (globalUmsetzung.getUmsetzungsModul() != null)
        {
          ergebnis = umsetzungHelligkeitAbarbeiten(globalUmsetzung.getUmsetzungsModul(), resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, i);
  
          if (ergebnis != null)
          {
            datenVerarbeitet = true;
            
            break;
          }
        }
      }
        
      if (!datenVerarbeitet)
      {
        ergebnis = umsetzungHelligkeitAbarbeiten(klassenNameDefault, resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, -1);
      }
  
      if (ergebnis != null)
      {
        ergebnis.setDataTime(zeitpunkt.getTime());

        try
        {
          m_debug.fine("Senden Helligkeit " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnitt().getPid());
          
          m_connection.sendData(ergebnis);
        }
        catch (DataNotSubscribedException e)
        {
          DataDescription dd = ergebnis.getDataDescription();
          List<SystemObject> objekte = new ArrayList<SystemObject>();
          objekte.add(ergebnis.getObject());
          
          try
          {
            m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());

            m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
          }
          catch (OneSubscriptionPerSendData e2)
          {
            e2.printStackTrace();
          } 

          try
          {
            Thread.sleep(500);
          }
          catch (InterruptedException e2)
          {
            e2.printStackTrace();
          }
          
          try
          {
            m_connection.sendData(ergebnis);
          }
          catch (DataNotSubscribedException e1)
          {
            e1.printStackTrace();
          }
          catch (SendSubscriptionNotConfirmed e1)
          {
            m_debug.warning("Ich 2", e1);
            
//            e1.printStackTrace();
          }
        }
        catch (SendSubscriptionNotConfirmed e)
        {
          m_debug.warning("Ich 1", e);
          
//          e.printStackTrace();
        }
      }
    }

    // Betriebsart umsetzen
    
    if (anzeigeQuerschnittAktorUms != null) 
//    if ((anzeigeQuerschnittAktorUms != null) && (anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getListeUmsetzungBetriebsart().size() > 0))
    {
      m_debug.fine("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + resultData.toString());
//      m_debug.error("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + result.toString());

      SystemObject object = resultData.getObject();
      
      AttributeGroup atg = resultData.getDataDescription().getAttributeGroup();
      Aspect         asp = resultData.getDataDescription().getAspect();

      AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();
      
      int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungBetriebsart().size();

      for (int i = 0; i < anzahlGlobalUmsetzung; i++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungBetriebsart().get(i);
          
        if (globalUmsetzung.getUmsetzungsModul() != null)
        {
          ergebnis = umsetzungBetriebsartAbarbeiten(globalUmsetzung.getUmsetzungsModul(), resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, i);
  
          if (ergebnis != null)
          {
            datenVerarbeitet = true;
            
            break;
          }
        }
      }
        
      if (!datenVerarbeitet)
      {
        ergebnis = umsetzungBetriebsartAbarbeiten(klassenNameDefault, resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, -1);
      }
  
      if (ergebnis != null)
      {
        ergebnis.setDataTime(zeitpunkt.getTime());

        try
        {
          m_debug.fine("Senden Betriebsart " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnitt().getPid());
          
          m_connection.sendData(ergebnis);
        }
        catch (DataNotSubscribedException e)
        {
          DataDescription dd = ergebnis.getDataDescription();
          List<SystemObject> objekte = new ArrayList<SystemObject>();
          objekte.add(ergebnis.getObject());
          
          try
          {
            m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());

            m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
          }
          catch (OneSubscriptionPerSendData e2)
          {
            e2.printStackTrace();
          } 

          try
          {
            Thread.sleep(500);
          }
          catch (InterruptedException e2)
          {
            e2.printStackTrace();
          }
          
          try
          {
            m_connection.sendData(ergebnis);
          }
          catch (DataNotSubscribedException e1)
          {
            e1.printStackTrace();
          }
          catch (SendSubscriptionNotConfirmed e1)
          {
            m_debug.warning("Ich 5", e1);

//            e1.printStackTrace();
          }
        }
        catch (SendSubscriptionNotConfirmed e)
        {
          m_debug.warning("Ich 6", e);

//          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Abarbeitung der Stromausfall-Information.
   * 
   * @param resultData Ergebnisdaten der DE.
   * @param anzeigeQuerschnitt betroffener AnzeigeQuerschnitt.
   * @param de auslsende DE.
   * @param stromausfall <code>true</code>, wenn Stromausfall vorliegt, sonst <code>false</code>.
   */
  private void anzeigeQuerschnittStromausfallVerarbeiten(ResultData resultData, SystemObject anzeigeQuerschnitt, SystemObject de, boolean stromausfall)
  {
    ResultData ergebnis = null;

    ergebnis = umsetzungStromausfallAbarbeiten(klassenNameDefault, anzeigeQuerschnitt, de, stromausfall);
    
    m_debug.fine("Stromausfall " + ergebnis);

    if (ergebnis != null)
    {
      ergebnis.setDataTime(resultData.getDataTime());

      try
      {
        m_debug.fine("Senden Stromausfall " + anzeigeQuerschnitt.getPid());
        
        m_connection.sendData(ergebnis);
      }
      catch (DataNotSubscribedException e)
      {
        m_debug.warning("Stromausfall 1", e);
      }
      catch (SendSubscriptionNotConfirmed e)
      {
        m_debug.warning("Stromausfall 2", e);
      }
    }
  }
  
  /**
   * Abarbeitung der Trkontakt-Information.
   * 
   * @param resultData Ergebnisdaten der DE.
   * @param anzeigeQuerschnitt betroffener AnzeigeQuerschnitt.
   * @param de auslsende DE.
   * @param tuerkontakt <code>true</code>, wenn Tr offen vorliegt, sonst <code>false</code>.
   */
  private void anzeigeQuerschnittTuerkontaktVerarbeiten(ResultData resultData, SystemObject anzeigeQuerschnitt, SystemObject de, boolean tuerkontakt)
  {
    ResultData ergebnis = null;

    ergebnis = umsetzungTuerkontaktAbarbeiten(klassenNameDefault, anzeigeQuerschnitt, de, tuerkontakt);
    
    m_debug.fine("Trkontakt " + ergebnis);

    if (ergebnis != null)
    {
      ergebnis.setDataTime(resultData.getDataTime());

      try
      {
        m_debug.fine("Senden Trkontakt " + anzeigeQuerschnitt.getPid());
        
        m_connection.sendData(ergebnis);
      }
      catch (DataNotSubscribedException e)
      {
        m_debug.warning("Trkontakt 1", e);
      }
      catch (SendSubscriptionNotConfirmed e)
      {
        m_debug.warning("Trkontakt 2", e);
      }
    }
  }
  
  /**
   * Methode fr die Abarbeitung der Umsetzung der Helligkeit fr den AnzeigeQuerschnitt.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param zeitpunktUebergeordnet bergeordneter Schaltzeitpunkt.
   * @param object Anzeige
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param anzeigeQuerschnittTypUmsetzungTls AnzeigeQuerschnittTypUmsetzungTls fr diese Anzeige.
   * @param indexUmsetzung Index der Umsetzung.
   * @return Befehle fr das Teilmodell KExTls.
   */
  public ResultData umsetzungHelligkeitAbarbeiten(String klassenName, ResultData result, Long zeitpunktUebergeordnet, SystemObject object, AttributeGroup atg, Aspect asp, AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls, int indexUmsetzung)
  {
//    System.out.println("AnzeigeQuerschnitt.umsetzungHelligkeitAbarbeiten()");
    
    ResultData ergebnis = null;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeQuerschnittUmsetzungHelligkeitAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);

//      System.out.println("klasse " + klasse);
      
      Class[] formaleParameter = new Class[5];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
      formaleParameter[4] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyDes(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittEigenschaftIst(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittSchaltbarkeitNba(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittStatus()});
      
      Method methode2 = null;
      
      for (int i = 0; i < methodenHelligkeit.length; i++)
      {
        if (atg.getPid().equals(methodenHelligkeit[i][0]))
        {
          aktuelleMethode = methodenHelligkeit[i][1];
          
//          System.out.println("methode " + aktuelleMethode);
          
          methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class, Long.class});
          
          break;
        }
      }
      
      if (methode2 != null)
      {
        ergebnis = (ResultData)methode2.invoke(ret, new Object[]{result, zeitpunktUebergeordnet});
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der Helligkeitsumsetzung konnte nicht geladen werden.");

      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }

    return ergebnis;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung der Betriebsart fr den AnzeigeQuerschnitt.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param zeitpunktUebergeordnet bergeordneter Schaltzeitpunkt.
   * @param object Anzeige
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param anzeigeQuerschnittTypUmsetzungTls AnzeigeQuerschnittTypUmsetzungTls fr diese Anzeige.
   * @param indexUmsetzung Index der Umsetzung.
   * @return Befehle fr das Teilmodell KExTls.
   */
  public ResultData umsetzungBetriebsartAbarbeiten(String klassenName, ResultData result, Long zeitpunktUebergeordnet, SystemObject object, AttributeGroup atg, Aspect asp, AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls, int indexUmsetzung)
  {
//    System.out.println("AnzeigeQuerschnitt.umsetzungBetriebsartAbarbeiten()");
    
    ResultData ergebnis = null;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeQuerschnittUmsetzungBetriebsartAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);

//      System.out.println("klasse " + klasse);
      
      Class[] formaleParameter = new Class[5];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
      formaleParameter[4] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyDes(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittEigenschaftIst(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittSchaltbarkeitNba(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittStatus()});
      
      Method methode2 = null;
      
      for (int i = 0; i < methodenBetriebsart.length; i++)
      {
        if (atg.getPid().equals(methodenBetriebsart[i][0]))
        {
          aktuelleMethode = methodenBetriebsart[i][1];

//          System.out.println("methode " + aktuelleMethode);
          
          methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class, Long.class});
          
          break;
        }
      }
      
      if (methode2 != null)
      {
        ergebnis = (ResultData)methode2.invoke(ret, new Object[]{result, zeitpunktUebergeordnet});
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der Betriebsartumsetzung konnte nicht geladen werden.");
      
      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }

    return ergebnis;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung des Stromausfall fr den AnzeigeQuerschnitt.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param anzeigeQuerschnitt betroffener AnzeigeQuerschnitt.
   * @param de auslsende DE.
   * @param stromausfall <code>true</code>, wenn Stromausfall vorliegt, sonst <code>false</code>.
   * @return ATG atg.anzeigeQuerschnittEigenschaftIst.
   */
  public ResultData umsetzungStromausfallAbarbeiten(String klassenName, SystemObject anzeigeQuerschnitt, SystemObject de, Boolean stromausfall)
  {
    ResultData ergebnis = null;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeQuerschnittStromausfallAbarbeiten " + klassenName + " anzeigeQuerschnitt " + anzeigeQuerschnitt + " de " + de + " Stromausfall " + stromausfall);

    try
    {
      klasse = Class.forName(klassenName);

      Class[] formaleParameter = new Class[5];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
      formaleParameter[4] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyDes(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittEigenschaftIst(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittSchaltbarkeitNba(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittStatus()});
      
      Method methode2 = null;
      
      aktuelleMethode = methodeStromausfall;
          
      methode2 = klasse.getMethod(aktuelleMethode, new Class[]{SystemObject.class, SystemObject.class, Boolean.class});
      
      if (methode2 != null)
      {
        ergebnis = (ResultData)methode2.invoke(ret, new Object[]{anzeigeQuerschnitt, de, stromausfall});
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten des Stromausfalls konnte nicht geladen werden.");

      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }

    return ergebnis;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung des Trkontakt fr den AnzeigeQuerschnitt.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param anzeigeQuerschnitt betroffener AnzeigeQuerschnitt.
   * @param de auslsende DE.
   * @param tuerkontakt <code>true</code>, wenn Tr offen vorliegt, sonst <code>false</code>.
   * @return ATG atg.anzeigeQuerschnittEigenschaftIst.
   */
  public ResultData umsetzungTuerkontaktAbarbeiten(String klassenName, SystemObject anzeigeQuerschnitt, SystemObject de, Boolean tuerkontakt)
  {
    ResultData ergebnis = null;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeQuerschnittTuerkontaktAbarbeiten " + klassenName + " anzeigeQuerschnitt " + anzeigeQuerschnitt + " de " + de + " Trkontakt " + tuerkontakt);

    try
    {
      klasse = Class.forName(klassenName);

      Class[] formaleParameter = new Class[5];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
      formaleParameter[4] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyDes(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittEigenschaftIst(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittSchaltbarkeitNba(),
                                                        m_zwischenschichtAq.getListeAnzeigeQuerschnittStatus()});
      
      Method methode2 = null;
      
      aktuelleMethode = methodeTuerkontakt;
          
      methode2 = klasse.getMethod(aktuelleMethode, new Class[]{SystemObject.class, SystemObject.class, Boolean.class});
      
      if (methode2 != null)
      {
        ergebnis = (ResultData)methode2.invoke(ret, new Object[]{anzeigeQuerschnitt, de, tuerkontakt});
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten des Trkontakts konnte nicht geladen werden.");

      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }

    return ergebnis;
  }

  /**
   * Verarbeiten der Befehle fr die Anzeige.
   * 
   * @param result Befehle fr die Anzeige.
   */
  private void anzeigeEigenschaftSollVerarbeiten(ResultData result)
  {
    SystemObject objekt = result.getObject();
    
    String pidAnzeige = objekt.getPid();
    
    // Eingangspuffer auf vorhandene Daten prfen
    
    int index = -1;
    
    for (int i = 0; i < getEingangsPufferAnzeigeBefehle().size(); i++)
    {
      if (getEingangsPufferAnzeigeBefehle().get(i).getObject().getPid().equals(pidAnzeige))
      {
        index = i;
        
        break;
      }
    }
    
    if (index != -1) // wenn schon Daten im Eingangspuffer vorhanden, dann entfernen
    {
      getEingangsPufferAnzeigeBefehle().removeElementAt(index);
    }
    
    if (result.hasData()) // wenn Daten vorhanden, hinzufgen
    {
      getEingangsPufferAnzeigeBefehle().add(result);
    }
  }

  /**
   * Verarbeiten der Befehle fr die Anzeige.
   * 
   * @param result Befehle fr die Anzeige.
   */
  private void anzeigeAutarkSollVerarbeiten(ResultData result)
  {
    SystemObject objekt = result.getObject();
    
    String pidAnzeige = objekt.getPid();
    
    Vector<ResultData> atgSenden = new Vector<ResultData>();
    
    for (int k = 0; k < m_anzeigen.size(); k++)
    {
      Anzeige anzeige = m_anzeigen.get(k);
      
      if (anzeige.getPid().equals(objekt.getPid()))
      {
        ResultData result1 = anzeige.anzeigeVerarbeiten(result, new Long(new Date().getTime()));
        
        if (result1 != null)
        {
          atgSenden.add(result1);
        }
        
//        System.out.println("result1 " + result1);
      }
    }

    Date zeitpunkt = new Date();
    
    for (int i = 0; i < atgSenden.size(); i++)
    {
      ResultData res = atgSenden.get(i);        
      
      res.setDataTime(zeitpunkt.getTime());
      
      try
      {
//        System.out.println("Senden " + atgSenden.get(i));
        
        m_connection.sendData(res);
      }
      catch (DataNotSubscribedException e)
      {
        DataDescription dd = res.getDataDescription();
        List<SystemObject> objekte = new ArrayList<SystemObject>();
        objekte.add(res.getObject());
        
        try
        {
          m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());

          m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
        }
        catch (OneSubscriptionPerSendData e2)
        {
          e2.printStackTrace();
        } 

        try
        {
          Thread.sleep(500);
        }
        catch (InterruptedException e2)
        {
          e2.printStackTrace();
        }
        
        try
        {
          m_connection.sendData(res);
        }
        catch (DataNotSubscribedException e1)
        {
          e1.printStackTrace();
        }
        catch (SendSubscriptionNotConfirmed e1)
        {
          m_debug.warning("Ich 3", e1);
//          e1.printStackTrace();
        }
      }
      catch (SendSubscriptionNotConfirmed e)
      {
        m_debug.warning("Ich 4", e);

//        e.printStackTrace();
      }
    }
    
    
    
    
//    // Eingangspuffer auf vorhandene Daten prfen
//    
//    int index = -1;
//    
//    for (int i = 0; i < getEingangsPufferAutarkBefehle().size(); i++)
//    {
//      if (getEingangsPufferAutarkBefehle().get(i).getObject().getPid().equals(pidAnzeige))
//      {
//        index = i;
//        
//        break;
//      }
//    }
//    
//    if (index != -1) // wenn schon Daten im Eingangspuffer vorhanden, dann entfernen
//    {
//      getEingangsPufferAutarkBefehle().removeElementAt(index);
//    }
//    
//    if (result.hasData()) // wenn Daten vorhanden, hinzufgen
//    {
//      getEingangsPufferAutarkBefehle().add(result);
//    }
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientSenderInterface#dataRequest(stauma.dav.configuration.interfaces.SystemObject, stauma.dav.clientside.DataDescription, byte)
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientSenderInterface#isRequestSupported(stauma.dav.configuration.interfaces.SystemObject, stauma.dav.clientside.DataDescription)
   */
  public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
  {
    return false;
  }

  /**
   * Holen der Pid des AnzeigeQuerschnitt.
   * 
   * @return Pid des AnzeigeQuerschnitt.
   */
  public String getPid()
  {
    return m_anzeigeQuerschnitt.getPid();
  }

  /**
   * Holen des Dirigenten, dem der AnzeigeQuerschnitt angehrt.
   * 
   * @return Dirigent, dem der AnzeigeQuerschnitt angehrt.
   */
  public Dirigent getDirigent()
  {
    return m_dirigent;
  }

  /**
   * Setzen des Dirigenten, dem der AnzeigeQuerschnitt angehrt.
   * 
   * @param dirigent Dirigent, dem der AnzeigeQuerschnitt angehrt.
   */
  public void setDirigent(Dirigent dirigent)
  {
    m_dirigent = dirigent;
  }

  /**
   * Holen des Empfngers fr die Befehle fr den AnzeigeQuerschnitt.
   * 
   * @return Empfnger fr die Befehle fr den AnzeigeQuerschnitt.
   */
  public ClientReceiverInterface getEmpfaenger()
  {
    ClientReceiverInterface empfaenger = this;
    
    if (getDirigent() != null)
    {
      empfaenger = getDirigent().getEmpfaenger();
    }
    
    return empfaenger;
  }

  /**
   * Holen des Senders fr die Befehle fr den AnzeigeQuerschnitt.
   * 
   * @return Sender fr die Befehle fr den AnzeigeQuerschnitt.
   */
  public ClientSenderInterface getSender()
  {
    ClientSenderInterface sender = this;
    
    return sender;
  }

  /**
   * Anmelden zum Datenempfang fr den Empfnger.
   */
  public void anmeldungDatenempfang()
  {
    anmeldungDatenempfang(getEmpfaenger());

    if (m_zusaetzlichAnzumeldendeDaten.size() > 0)
    {
      Set<SystemObject> keySet = m_zusaetzlichAnzumeldendeDaten.keySet();
      
      for (SystemObject key : keySet)
      {
        DataDescription dataDescription = m_zusaetzlichAnzumeldendeDaten.get(key);
        
        try
        {
          m_connection.subscribeReceiver(this, key, dataDescription, ReceiveOptions.delta(), ReceiverRole.receiver());
        }
        catch (NullPointerException e)
        {
          e.printStackTrace();
          
          System.exit(0);
        }
      }
    }
  }

  /**
   * Anmelden zum Datenversand fr den Sender.
   */
  public void anmeldungDatenversand()
  {
    anmeldungDatenversand(getSender());
  }

  /**
   * Senden von Daten ber den Sender.
   * 
   * @param resultData zu versendende Daten.
   * @param zeitpunkt bergeordneter Schaltzeitpunkt.
   * @return <code>true</code>.
   */
  public boolean datenSenden(ResultData resultData, long zeitpunkt)
  {
    if (zeitpunkt != 0)
    {
      resultData.setDataTime(zeitpunkt);
    }
        
    try
    {
      m_connection.sendData(resultData);
    }
    catch (DataNotSubscribedException e)
    {
      e.printStackTrace();
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      e.printStackTrace();
    }
    
    return true;
  }

  /**
   * Holen des Puffer fr Eingangsdaten bis zur Synchronisierung.
   * 
   * @return Puffer fr Eingangsdaten bis zur Synchronisierung.
   */
  public Vector<ResultData> getEingangsPufferAnzeigeBefehle()
  {
    return m_eingangsPufferAnzeigeBefehle;
  }

  /**
   * Holen des Puffer fr Eingangsdaten bis zur Synchronisierung.
   * 
   * @return Puffer fr Eingangsdaten bis zur Synchronisierung.
   */
  public Vector<ResultData> getEingangsPufferAutarkBefehle()
  {
    return m_eingangsPufferAutarkBefehle;
  }
}