/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls.anzeigenglobal;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;


import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittAktorTls;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittTypUmsetzungTls;
import de.bsvrz.ste.agzs.tls.DirigentAktorTls;
import de.bsvrz.ste.agzs.tls.DirigentUmsetzungTls;
import de.bsvrz.ste.agzs.tls.GlobalUmsetzung;
import de.bsvrz.ste.agzs.tls.AnzeigenGlobalZwischenSchichtTls;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.dirigent 
 * und die Abarbeitung der ber diesen Typ enpfangenen spezifischen Attributgruppen.
 * Die Abarbeitung erfolgt mit den versorgten Umsetzungsklassen, oder wenn keine spezielle versorgt, mit der Standardumsetzung.<br>
 * Die Standardumsetzungsklasse ist die Klasse 'zswzg.kex.tls.DirigentUmsetzungTlsDefault'.<br>
 * <br>
 * Abarbeitung: AnzeigenGlobal -> KExTls.<br>
 * <br>
 * Noch nicht vollstndig implementiert, da Funktionalitt derzeit nicht notwendig und nicht testbar.
 *
 * @version $Revision: 1.3 $ / $Date: 2012/03/09 10:55:21 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class Dirigent implements ClientReceiverInterface, ClientSenderInterface
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Konfigurationsobjekt vom Typ typ.dirigent.
   */
  private ConfigurationObject m_dirigent;
  /**
   * Datenverteilerverbindung
   */
  private ClientDavInterface m_connection;
  /**
   * Datenmodell.
   */
  private DataModel m_dataModel;
  /**
   * DirigentAktorTls mit Key Dirigent.
   */
  private HashMap<String, DirigentAktorTls> m_listeDirigentAktorTlsKeyDirigent;
  /**
   * Instanz der Klasse ZwischenschichtAq.
   */
  private AnzeigenGlobalZwischenSchichtTls m_zwischenschichtAq;
  /**
   * Liste mit AnzeigeQuerschnitte, die dem Dirigent angehren.
   */
  @SuppressWarnings("unused")
  private Vector<AnzeigeQuerschnitt> m_anzeigeQuerschnitte = new Vector<AnzeigeQuerschnitt>();
  /**
   * AttributGruppe-Aspekt-Kombinationen fr den Datenversand.
   */
  private String[][] m_datenversandAttributGruppeAspekt = 
  {
    {"atg.dirigentEigenschaftIst", "asp.aqZustand"},
  };
  /**
   * Puffer fr Eingangsdaten bis zur Synchronisierung.
   */
  private Vector<ResultData> m_eingangsPuffer = new Vector<ResultData>();
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung des Dirigenten.
   */
  private String[][] m_methoden =
  {
    {"atg.dirigentEigenschaftSoll", "dirigentEigenschaftSoll"},
  };
  
  /**
   * Konstruktor mit bergabe der notwendigen Parameter.
   * 
   * @param connection Datenverteilerverbindung.
   * @param dirigent Konfigurationsobjekt vom Typ typ.dirigent.
   * @param listeAnzeigeQuerschnittKlasse Liste mit AnzeigeQuerschnitten, die dem Dirigent angehren.
   */
  public Dirigent(ClientDavInterface connection, 
                  AnzeigenGlobalZwischenSchichtTls zwischenschichtAq, 
                  ConfigurationObject dirigent, 
                  Vector<AnzeigeQuerschnitt> listeAnzeigeQuerschnittKlasse,
                  HashMap<String, DirigentAktorTls> listeDirigentAktorTlsKeyDirigent)
  {
    m_connection = connection;
    m_zwischenschichtAq = zwischenschichtAq;
    m_dirigent = dirigent;
    m_listeDirigentAktorTlsKeyDirigent = listeDirigentAktorTlsKeyDirigent;
    m_dataModel = m_connection.getDataModel();

//    m_debug.error("AnzeigeQuerschnitt " + m_anzeigeQuerschnitt.getPid());
    
    List anzeigeQuerschnitte = m_dirigent.getNonMutableSet("AnzeigeQuerschnitte").getElements();
    
//    System.out.println("Dirigent " + m_dirigent.getPid());
    
    for (int i = 0; i < anzeigeQuerschnitte.size(); i++)
    {
      String pidAnzeigeQuerschnitt = ((SystemObject)anzeigeQuerschnitte.get(i)).getPid();

//      System.out.println("- AnzeigeQuerschnitt " + pidAnzeigeQuerschnitt);
      
      for (int j = 0; j < listeAnzeigeQuerschnittKlasse.size(); j++)
      {
        AnzeigeQuerschnitt anzeigeQuerschnittTmp = listeAnzeigeQuerschnittKlasse.get(j);
        
        if (pidAnzeigeQuerschnitt.equals(anzeigeQuerschnittTmp.getPid()))
        {
//          System.out.println("--- gefunden " + anzeigeQuerschnittTmp.getPid());
  
          // TODO aktuell soll noch ber AQ syncronisiert werden, nicht ber Dirigent
          
//          anzeigeQuerschnittTmp.setDirigent(this);   
          
          break;
        }
      }
    }
  }

  /**
   * Anmelden fr den Datenversand.
   * 
   * @param sender Instanz des Senders.
   */
  protected void anmeldungDatenversand(ClientSenderInterface sender)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // Dirigent auf Datenversand anmelden

    for (int i = 0; i < m_datenversandAttributGruppeAspekt.length; i++)
    {
      atgDaten = m_dataModel.getAttributeGroup(m_datenversandAttributGruppeAspekt[i][0]);
      aspDaten = m_dataModel.getAspect(m_datenversandAttributGruppeAspekt[i][1]);
      ddDaten = new DataDescription(atgDaten, aspDaten);
      
      m_debug.config("Anmeldung Quelle " + m_dirigent + " " + ddDaten);
  
      try
      {
        m_connection.subscribeSender(sender, m_dirigent, ddDaten, SenderRole.source());
      }
      catch (OneSubscriptionPerSendData e)
      {
        e.printStackTrace();
  
        System.exit(0);
      }
    }
  }

  /**
   * Abmelden vom Datenversand.
   * 
   * @param sender Instanz des Senders.
   */
  protected void abmeldungDatenversand(ClientSenderInterface sender)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // Dirigent vom Datenversand abmelden
        
    atgDaten = m_dataModel.getAttributeGroup("atg.dirigentEigenschaftIst");
    aspDaten = m_dataModel.getAspect("asp.aqZustand");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Abmeldung " + m_dirigent + " " + ddDaten);

    m_connection.unsubscribeSender(sender, m_dirigent, ddDaten);
  }

  /**
   * Abmelden vom Datenempfang.
   * 
   * @param empfaenger Instanz des Empfngers.
   */
  protected void anmeldungDatenempfang(ClientReceiverInterface empfaenger)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // Dirigent auf Datenempfang anmelden
    
    atgDaten = m_dataModel.getAttributeGroup("atg.dirigentEigenschaftSoll");
    aspDaten = m_dataModel.getAspect("asp.aqVorgabe");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Anmeldung Senke " + m_dirigent.getPid() + " " + ddDaten);

    try
    {
      m_connection.subscribeReceiver(empfaenger, m_dirigent, ddDaten, ReceiveOptions.normal(), ReceiverRole.drain());
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();

      System.exit(0);
    }
  }

  /**
   * Abmelden vom Datenempfang.
   * 
   * @param empfaenger Instanz des Empfngers.
   */
  protected void abmeldungDatenempfang(ClientReceiverInterface empfaenger)
  {
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // Dirigent vom Datenempfang abmelden
    
    atgDaten = m_dataModel.getAttributeGroup("atg.dirigentEigenschaftSoll");
    aspDaten = m_dataModel.getAspect("asp.aqVorgabe");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Abmeldung " + m_dirigent.getPid() + " " + ddDaten);

    m_connection.unsubscribeReceiver(empfaenger, m_dirigent, ddDaten);
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientReceiverInterface#update(stauma.dav.clientside.ResultData[])
   */
  public void update(ResultData[] results)
  {
    for (ResultData result : results)
    {
//      System.out.println("Dirigent Result " + result);
      
      String atg = result.getDataDescription().getAttributeGroup().getPid();
      
      if (atg.equals("atg.dirigentEigenschaftSoll"))
      {
        if (result.hasData())
        {
//          System.out.println("Schalten !!!");
          
          dirigentEigenschaftenSollVerarbeiten(result);
        }
        else // Eingangspuffer lschen
        {
          getEingangsPuffer().removeAllElements();
        }
      }
      else if (atg.equals("atg.anzeigeQuerschnittEigenschaftSoll"))
      {
        anzeigeQuerschnittEigenschaftenSollVerarbeiten(result);
      }
      else
      {
        if (atg.equals("atg.anzeigeEigenschaftSoll"))
        {
          anzeigeEigenschaftSollVerarbeiten(result);
        }
      }
    }
  }

  /**
   * Verarbeiten der Befehle fr den Dirigenten.
   * 
   * @param resultData Befehle fr den Dirigenten.
   */
  private void dirigentEigenschaftenSollVerarbeiten(ResultData resultData)
  {
    ResultData ergebnis = null;
    
    System.out.println("dirigentEigenschaftenSollVerarbeiten " + resultData);
  }

  /**
   * Verarbeiten der Befehle fr den AnzeigeQuerschnitt.
   * 
   * @param resultData Befehle fr den AnzeigeQuerschnitt.
   */
  private void anzeigeQuerschnittEigenschaftenSollVerarbeiten(ResultData resultData)
  {
    ResultData ergebnis = null;

    boolean datenVerarbeitet = false;
    
    Vector<ResultData> atgSenden = new Vector<ResultData>();
    
//    System.out.println("Folgende Anzeigen bearbeiten:");
    
    Data data = resultData.getData();
    
    long zeitpunktUebergeordnet = data.getTimeValue("SollZeitpunkt").getMillis();
    
    ReferenceArray array = data.getReferenceArray("Anzeigen");
    
//    for (int i = 0; i < array.getLength(); i++)
//    {
//      SystemObject objekt = array.getReferenceValue(i).getSystemObject();
//
//      System.out.println("- Anzeige " + objekt.getPid());
//    }

    for (int i = 0; i < array.getLength(); i++)
    {
      SystemObject objekt = array.getReferenceValue(i).getSystemObject();

      for (int j = 0; j < getEingangsPuffer().size(); j++)
      {
        ResultData resultDataSenden = getEingangsPuffer().get(j);
        
        if (resultDataSenden.getObject().getPid().equals(objekt.getPid()))
        {
          for (int k = 0; k < m_anzeigeQuerschnitte.size(); k++)
          {
            AnzeigeQuerschnitt anzeigeQuerschnitt = m_anzeigeQuerschnitte.get(k);
            
            if (anzeigeQuerschnitt.getPid().equals(objekt.getPid()))
            {
//              ResultData result1 = anzeigeQuerschnitt.anzeigeVerarbeiten(resultDataSenden, new Long(zeitpunktUebergeordnet));
//              
//              if (result1 != null)
//              {
//                atgSenden.add(result1);
//              }
              
//              System.out.println("result1 " + result1);
            }
          }
        }
      }
    }
    
//    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktorUms = (AnzeigeQuerschnittAktorTls)m_listeAnzeigeQuerschnittAktorTlsKeyAnzeige.get(getPid());
//    
//    if (anzeigeQuerschnittAktorUms != null)
//    {
//      SystemObject eak = anzeigeQuerschnittAktorUms.getEak();
//
//      Data dataEak = m_connection.createData(m_dataModel.getAttributeGroup("atg.tlsEinzelTelegrammSenden"));
//      
//      dataEak.getScaledValue("Senden").setText("senden");
//      
//      AttributeGroup atgEakErgebnis = ZwischenschichtAq.getConnection().getDataModel().getAttributeGroup("atg.tlsEinzelTelegrammSenden");
//      Aspect aspEakErgebnis = ZwischenschichtAq.getConnection().getDataModel().getAspect("asp.tlsVorgabe");
//      
//      ResultData resultEak = new ResultData(eak, new DataDescription(atgEakErgebnis, aspEakErgebnis), zeitpunktUebergeordnet, dataEak);
//
//      atgSenden.add(resultEak);
//
//      System.out.println("resultEak " + resultEak);
//    }
//    
//    Date zeitpunkt = new Date();
//    
//    for (int i = 0; i < atgSenden.size(); i++)
//    {
//      ResultData res = atgSenden.get(i);        
//      
//      res.setDataTime(zeitpunkt.getTime());
//      
//      try
//      {
//        System.out.println("Senden " + atgSenden.get(i));
//        
//        m_connection.sendData(res);
//      }
//      catch (DataNotSubscribedException e)
//      {
//        DataDescription dd = res.getDataDescription();
//        List<SystemObject> objekte = new ArrayList<SystemObject>();
//        objekte.add(res.getObject());
//        
//        try
//        {
//          m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());
//
//          m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
//        }
//        catch (OneSubscriptionPerSendData e2)
//        {
//          e2.printStackTrace();
//        } 
//
//        try
//        {
//          Thread.sleep(500);
//        }
//        catch (InterruptedException e2)
//        {
//          e2.printStackTrace();
//        }
//        
//        try
//        {
//          m_connection.sendData(res);
//        }
//        catch (DataNotSubscribedException e1)
//        {
//          e1.printStackTrace();
//        }
//        catch (SendSubscriptionNotConfirmed e1)
//        {
//          m_debug.warning("Ich 3", e1);
////          e1.printStackTrace();
//        }
//      }
//      catch (SendSubscriptionNotConfirmed e)
//      {
//        m_debug.warning("Ich 4", e);
//
////        e.printStackTrace();
//      }
//    }
//    
//    getEingangsPuffer().removeAllElements();
//
//    // Helligkeit umsetzen
//    
//    if (anzeigeQuerschnittAktorUms != null)
////    if ((anzeigeQuerschnittAktorUms != null) && (anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getListeUmsetzungHelligkeit().size() > 0))
//    {
//      m_debug.fine("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + resultData.toString());
////      m_debug.error("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + result.toString());
//
//      SystemObject object = resultData.getObject();
//      
//      AttributeGroup atg = resultData.getDataDescription().getAttributeGroup();
//      Aspect         asp = resultData.getDataDescription().getAspect();
//
//      AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();
//      
//      int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungHelligkeit().size();
//
//      for (int i = 0; i < anzahlGlobalUmsetzung; i++)
//      {
//        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungHelligkeit().get(i);
//          
//        if (globalUmsetzung.getUmsetzungsModul() != null)
//        {
//          ergebnis = umsetzungHelligkeitAbarbeiten(globalUmsetzung.getUmsetzungsModul(), resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, i);
//  
//          if (ergebnis != null)
//          {
//            datenVerarbeitet = true;
//            
//            break;
//          }
//        }
//      }
//        
//      if (!datenVerarbeitet)
//      {
//        ergebnis = umsetzungHelligkeitAbarbeiten(klassenNameDefault, resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, -1);
//      }
//  
//      if (ergebnis != null)
//      {
//        ergebnis.setDataTime(zeitpunkt.getTime());
//
//        try
//        {
//          m_debug.fine("Senden Helligkeit " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnitt().getPid());
//          
//          m_connection.sendData(ergebnis);
//        }
//        catch (DataNotSubscribedException e)
//        {
//          DataDescription dd = ergebnis.getDataDescription();
//          List<SystemObject> objekte = new ArrayList<SystemObject>();
//          objekte.add(ergebnis.getObject());
//          
//          try
//          {
//            m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());
//
//            m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
//          }
//          catch (OneSubscriptionPerSendData e2)
//          {
//            e2.printStackTrace();
//          } 
//
//          try
//          {
//            Thread.sleep(500);
//          }
//          catch (InterruptedException e2)
//          {
//            e2.printStackTrace();
//          }
//          
//          try
//          {
//            m_connection.sendData(ergebnis);
//          }
//          catch (DataNotSubscribedException e1)
//          {
//            e1.printStackTrace();
//          }
//          catch (SendSubscriptionNotConfirmed e1)
//          {
//            m_debug.warning("Ich 2", e1);
//            
////            e1.printStackTrace();
//          }
//        }
//        catch (SendSubscriptionNotConfirmed e)
//        {
//          m_debug.warning("Ich 1", e);
//          
////          e.printStackTrace();
//        }
//      }
//    }
//
//    // Betriebsart umsetzen
//    
//    if (anzeigeQuerschnittAktorUms != null) 
////    if ((anzeigeQuerschnittAktorUms != null) && (anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getListeUmsetzungBetriebsart().size() > 0))
//    {
//      m_debug.fine("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + resultData.toString());
////      m_debug.error("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + result.toString());
//
//      SystemObject object = resultData.getObject();
//      
//      AttributeGroup atg = resultData.getDataDescription().getAttributeGroup();
//      Aspect         asp = resultData.getDataDescription().getAspect();
//
//      AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();
//      
//      int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungBetriebsart().size();
//
//      for (int i = 0; i < anzahlGlobalUmsetzung; i++)
//      {
//        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeUmsetzungBetriebsart().get(i);
//          
//        if (globalUmsetzung.getUmsetzungsModul() != null)
//        {
//          ergebnis = umsetzungBetriebsartAbarbeiten(globalUmsetzung.getUmsetzungsModul(), resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, i);
//  
//          if (ergebnis != null)
//          {
//            datenVerarbeitet = true;
//            
//            break;
//          }
//        }
//      }
//        
//      if (!datenVerarbeitet)
//      {
//        ergebnis = umsetzungBetriebsartAbarbeiten(klassenNameDefault, resultData, zeitpunktUebergeordnet, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, -1);
//      }
//  
//      if (ergebnis != null)
//      {
//        ergebnis.setDataTime(zeitpunkt.getTime());
//
//        try
//        {
//          m_debug.fine("Senden Betriebsart " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnitt().getPid());
//          
//          m_connection.sendData(ergebnis);
//        }
//        catch (DataNotSubscribedException e)
//        {
//          DataDescription dd = ergebnis.getDataDescription();
//          List<SystemObject> objekte = new ArrayList<SystemObject>();
//          objekte.add(ergebnis.getObject());
//          
//          try
//          {
//            m_connection.subscribeSender(this, objekte, dd, SenderRole.sender());
//
//            m_debug.finest("Datenanmeldung erfolgt " + objekte + " " + dd);
//          }
//          catch (OneSubscriptionPerSendData e2)
//          {
//            e2.printStackTrace();
//          } 
//
//          try
//          {
//            Thread.sleep(500);
//          }
//          catch (InterruptedException e2)
//          {
//            e2.printStackTrace();
//          }
//          
//          try
//          {
//            m_connection.sendData(ergebnis);
//          }
//          catch (DataNotSubscribedException e1)
//          {
//            e1.printStackTrace();
//          }
//          catch (SendSubscriptionNotConfirmed e1)
//          {
//            m_debug.warning("Ich 5", e1);
//
////            e1.printStackTrace();
//          }
//        }
//        catch (SendSubscriptionNotConfirmed e)
//        {
//          m_debug.warning("Ich 6", e);
//
////          e.printStackTrace();
//        }
//      }
//    }
  }

  /**
   * Verarbeiten der Befehle fr die Anzeige.
   * 
   * @param result Befehle fr die Anzeige.
   */
  private void anzeigeEigenschaftSollVerarbeiten(ResultData result)
  {
    SystemObject objekt = result.getObject();
    
    String pidAnzeige = objekt.getPid();
    
    // Eingangspuffer auf vorhandene Daten prfen
    
    int index = -1;
    
    for (int i = 0; i < getEingangsPuffer().size(); i++)
    {
      if (getEingangsPuffer().get(i).getObject().getPid().equals(pidAnzeige))
      {
        index = i;
        
        break;
      }
    }
    
    if (index != -1) // wenn schon Daten im Eingangspuffer vorhanden, dann entfernen
    {
      getEingangsPuffer().removeElementAt(index);
    }
    
    if (result.hasData()) // wenn Daten vorhanden, hinzufgen
    {
      getEingangsPuffer().add(result);
    }
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientSenderInterface#dataRequest(stauma.dav.configuration.interfaces.SystemObject, stauma.dav.clientside.DataDescription, byte)
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientSenderInterface#isRequestSupported(stauma.dav.configuration.interfaces.SystemObject, stauma.dav.clientside.DataDescription)
   */
  public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
  {
    return false;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung fr den Dirigenten.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param zeitpunktUebergeordnet bergeordneter Schaltzeitpunkt.
   * @param object Dirigent
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param dirigentUmsetzungTls DirigentUmsetzungTls fr diesen Dirigenten.
   * @param indexUmsetzung Index der Umsetzung.
   * @return Befehle fr das Teilmodell KExTls.
   */
  public ResultData umsetzungAbarbeiten(String klassenName, ResultData result, Long zeitpunktUebergeordnet, SystemObject object, AttributeGroup atg, Aspect asp, DirigentUmsetzungTls dirigentUmsetzungTls, int indexUmsetzung)
  {
    ResultData ergebnis = null;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("dirigentUmsetzungAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);

//      System.out.println("klasse " + klasse);
      
      Class[] formaleParameter = new Class[4];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = boolean.class;
      formaleParameter[3] = boolean.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
//      Object ret = construktor.newInstance(new Object[]{m_listeDirigentAktorTlsKeyDes, m_listeDirigentAktorTlsKeyDirigent});
      Object ret = null;
      
      Method methode2 = null;
      
      for (int i = 0; i < m_methoden.length; i++)
      {
        if (atg.getPid().equals(m_methoden[i][0]))
        {
          aktuelleMethode = m_methoden[i][1];
          
//          System.out.println("methode " + aktuelleMethode);
          
          methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class, Long.class});
          
          break;
        }
      }
      
      if (methode2 != null)
      {
        ergebnis = (ResultData)methode2.invoke(ret, new Object[]{result, zeitpunktUebergeordnet});
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der Anzeigenumsetzung konnte nicht geladen werden.");
      
      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
//    catch (InstantiationException e)
//    {
//      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");
//
//      e.printStackTrace();
//    }

    return ergebnis;
  }

  /**
   * Holen der Pid des Dirigenten.
   * 
   * @return Pid des Dirigenten.
   */
  public String getPid()
  {
    return m_dirigent.getPid();
  }

  /**
   * Holen des Empfngers fr die Befehle fr den Dirigenten.
   * 
   * @return Empfnger fr die Befehle fr den Dirigenten.
   */
  public ClientReceiverInterface getEmpfaenger()
  {
    ClientReceiverInterface empfaenger = this;
    
    return empfaenger;
  }

  /**
   * Holen des Senders fr die Befehle fr den Dirigenten.
   * 
   * @return Sender fr die Befehle fr den Dirigenten.
   */
  public ClientSenderInterface getSender()
  {
    ClientSenderInterface sender = this;
    
    return sender;
  }

  /**
   * Anmelden zum Datenempfang fr den Empfnger.
   */
  public void anmeldungDatenempfang()
  {
    anmeldungDatenempfang(getEmpfaenger());
  }

  /**
   * Anmelden zum Datenversand fr den Sender.
   */
  public void anmeldungDatenversand()
  {
    anmeldungDatenversand(getSender());
  }

  /**
   * Senden von Daten ber den Sender.
   * 
   * @param resultData zu versendende Daten.
   * @param zeitpunkt bergeordneter Schaltzeitpunkt.
   * @return <code>true</code>.
   */
  public boolean datenSenden(ResultData resultData, long zeitpunkt)
  {
    if (zeitpunkt != 0)
    {
      resultData.setDataTime(zeitpunkt);
    }
    
    try
    {
      m_connection.sendData(resultData);
    }
    catch (DataNotSubscribedException e)
    {
      e.printStackTrace();
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      e.printStackTrace();
    }
    
    return true;
  }

  /**
   * Holen des Puffer fr Eingangsdaten bis zur Synchronisierung.
   * 
   * @return Puffer fr Eingangsdaten bis zur Synchronisierung.
   */
  public Vector<ResultData> getEingangsPuffer()
  {
    return m_eingangsPuffer;
  }
}
