/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls.kex;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.agzs.tls.AnzeigeAktorTls;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittAktorTls;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittTypUmsetzungTls;
import de.bsvrz.ste.agzs.tls.AnzeigeTypUmsetzungTls;
import de.bsvrz.ste.agzs.tls.DirigentAktorTls;
import de.bsvrz.ste.agzs.tls.DirigentUmsetzungTls;
import de.bsvrz.ste.agzs.tls.GlobalUmsetzung;
import de.bsvrz.ste.agzs.tls.AnzeigenGlobalZwischenSchichtTls;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die Abarbeitung der von der KExTls empfangenen Attributgruppen.<br>
 * Die Abarbeitung erfolgt mit den versorgten Umsetzungsklassen, oder wenn keine spezielle versorgt, mit der Standardumsetzung.<br>
 * Fr die Anzeigen ist die Standardumsetzungsklasse die Klasse 'de.bsvrz.ste.agzs.tls.AnzeigeTypUmsetzungTlsDefault'.<br>
 * Fr die AnzeigeQuerschnitte ist die Standardumsetzungsklasse die Klasse 'de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittTypUmsetzungTlsDefault'.<br>
 * Das Ergebnis der Abarbeitung wird publiziert.<br>
 * <br>
 * Abarbeitung: KExTls -> AnzeigenGlobal.<br>
 *
 * @version $Revision: 1.5 $ / $Date: 2014/02/13 16:30:22 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class KexInputAnzeige extends KexInput implements ClientReceiverInterface, ClientSenderInterface
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug;
  /**
   * Datenverteilerverbindung.
   */
  private ClientDavInterface m_connection;
  /**
   * Instanz der Klasse ZwischenschichtAq.
   */
  private AnzeigenGlobalZwischenSchichtTls m_zwischenschichtAq;
  /**
   * Datenmodell.
   */
  private DataModel m_dataModel;
  /**
   * AnzeigeAktorTls mit Key DEs.
   */
  private HashMap m_listeAnzeigeAktorTlsDes;
  /**
   * AnzeigeAktorTls mit Key Anzeige.
   */
  private HashMap m_listeAnzeigeAktorTlsAnzeige;
  /**
   * Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   */
  private HashMap m_listeAnzeigeEigenschaftIst;
  /**
   * HashMap fr Status Methoden fr alle Anzeigen.
   */
  private HashMap m_listeAnzeigeStatus;
  /**
   * AnzeigeQuerschnittAktorTls mit Key DEs.
   */
  private HashMap m_listeAnzeigeQuerschnittAktorTlsDes;
  /**
   * AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   */
  private HashMap m_listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt;
  /**
   * Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   */
  private HashMap m_listeAnzeigeQuerschnittEigenschaftIst;
  /**
   * Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   */
  private HashMap m_listeAnzeigeQuerschnittSchaltbarkeitNba;
  /**
   * HashMap fr Status Methoden fr alle AnzeigenQuerschnitte.
   */
  private HashMap<String, HashMap> m_listeAnzeigeQuerschnittStatus;
  /**
   * Defaultklasse fr die AnzeigeTypUmsetzung.
   */
  private String klassenNameDefaultAnzeige = "de.bsvrz.ste.agzs.tls.AnzeigeTypUmsetzungTlsDefault";
  /**
   * Defaultklasse fr die AnzeigeQuerschnittTypUmsetzung.
   */
  private String klassenNameDefaultAnzeigeQuerschnitt = "de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittTypUmsetzungTlsDefault";
  /**
   * Defaultklasse fr die DirigentUmsetzung.
   */
  private String klassenNameDefaultDirigent = "de.bsvrz.ste.agzs.tls.DirigentUmsetzungTlsDefault";
  /**
   * DirigentAktorTls mit Key DEs.
   */
  private HashMap m_listeDirigentAktorTlsDes;
  /**
   * DirigentAktorTls mit Key Dirigent.
   */
  private HashMap m_listeDirigentAktorTlsDirigent;
  /**
   * Attributgruppe atg.dirigentEigenschaftIst fr alle Dirigenten aktuell.
   */
  private HashMap m_listeDirigentEigenschaftIst;
  /**
   * HashMap fr Status Methoden fr alle Dirigenten.
   */
  private HashMap<String, HashMap> m_listeDirigentStatus;
 
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung der Anzeige.
   */
  private String[][] m_anzeigeMethoden =
  {
    {"atg.tlsGloDeFehler", "asp.tlsAntwort", "tlsGloDeFehler", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsGloKanalSteuerung", "asp.parameterIst", "tlsGloKanalSteuerung", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgGestrteTextPositionen", "asp.tlsAntwort", "tlsWzgGestrteTextPositionen", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgNichtDarstellbareWVZ", "asp.tlsAntwort", "tlsWzgNichtDarstellbareWVZ", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgWechselText", "", "tlsWzgWechselText", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgWvzStellZustand", "", "tlsWzgWvzStellZustand", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgStellZustand", "", "tlsWzgStellZustand", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgDefekteLampen", "asp.tlsAntwort", "tlsWzgDefekteLampen", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgErgnzendeDeFehlerMeldung", "asp.tlsAntwort", "tlsWzgErgnzendeDeFehlerMeldung", "atg.anzeigeEigenschaftIst", "asp.aqZustand"},
    {"atg.tlsWzgGrundEinstellung", "asp.parameterIst", "tlsWzgGrundEinstellung", "atg.anzeigeAutarkIst", "asp.aqZustand"},
  };
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung der AnzeigeQuerschnitte.
   */
  private String[][] m_anzeigeQuerschnittMethoden =
  {
    {"atg.tlsGloDeFehler", "asp.tlsAntwort", "tlsGloDeFehler"},
    {"atg.tlsGloKanalSteuerung", "asp.parameterIst", "tlsGloKanalSteuerung"},
    {"atg.tlsWzgHelligkeit", "asp.tlsZustand", "tlsWzgHelligkeit"},
    {"atg.tlsWzgBetriebsArt", "asp.parameterIst", "tlsWzgBetriebsArt"},
    {"atg.tlsWzgBetriebsArt", "asp.tlsVorgabeBedienung", "tlsWzgBetriebsArtNba"},
  };
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung des Dirigenten.
   */
  private String[][] m_dirigentMethoden =
  {
    {"atg.tlsGloDeFehler", "asp.tlsAntwort", "tlsGloDeFehler"},
    {"atg.tlsGloKanalSteuerung", "asp.parameterIst", "tlsGloKanalSteuerung"},
    {"atg.tlsWzgWvzStellZustand", "", "tlsWzgWvzStellZustand"},
    {"atg.tlsWzgStellZustand", "", "tlsWzgStellZustand"},
  };
  
  /**
   * Konstruktor mit der bergabe der notwendigen Parameter.
   * 
   * @param connection Datenverteilerverbindung.
   * @param zwischenschichtAq Instanz der ZwischenschichtAq.
   * @param listeAnzeigeAktorTlsDes HashMap mit den AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige HashMap mit den AnzeigeAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst HashMap mit der Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus HashMap mit Status Methoden fr alle AnzeigenQuerschnitte.
   */
  public KexInputAnzeige(ClientDavInterface connection, 
                         AnzeigenGlobalZwischenSchichtTls zwischenschichtAq, 
                         HashMap listeAnzeigeAktorTlsDes, 
                         HashMap listeAnzeigeAktorTlsAnzeige, 
                         HashMap listeAnzeigeEigenschaftIst, 
                         HashMap listeAnzeigeStatus)
  {
    m_debug = Debug.getLogger();
    
    m_connection = connection;
    m_zwischenschichtAq = zwischenschichtAq;
    m_dataModel = connection.getDataModel();
    
    m_listeAnzeigeAktorTlsDes = listeAnzeigeAktorTlsDes;
    m_listeAnzeigeAktorTlsAnzeige = listeAnzeigeAktorTlsAnzeige;
    m_listeAnzeigeEigenschaftIst = listeAnzeigeEigenschaftIst;
    m_listeAnzeigeStatus = listeAnzeigeStatus;
    
    m_listeAnzeigeQuerschnittAktorTlsDes = m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyDes();
    m_listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt = m_zwischenschichtAq.getListeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt();
    m_listeAnzeigeQuerschnittEigenschaftIst = m_zwischenschichtAq.getListeAnzeigeQuerschnittEigenschaftIst();
    m_listeAnzeigeQuerschnittSchaltbarkeitNba = m_zwischenschichtAq.getListeAnzeigeQuerschnittSchaltbarkeitNba();
    m_listeAnzeigeQuerschnittStatus = m_zwischenschichtAq.getListeAnzeigeQuerschnittStatus();

    m_listeDirigentAktorTlsDes = m_zwischenschichtAq.getListeDirigentAktorTlsKeyDes();
    m_listeDirigentAktorTlsDirigent = m_zwischenschichtAq.getListeDirigentAktorTlsKeyDirigent();
    m_listeDirigentEigenschaftIst = m_zwischenschichtAq.getListeDirigentEigenschaftIst();
    m_listeDirigentStatus = m_zwischenschichtAq.getListeDirigentStatus();
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientReceiverInterface#update(stauma.dav.clientside.ResultData[])
   */
  public void update(ResultData[] results)
  {
    for(int resultIndex = 0; resultIndex < results.length; ++resultIndex) 
    {
      boolean datenVerarbeitet = false;
      
      ResultData result = results[resultIndex];
      SystemObject object = result.getObject();
      
      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      Aspect         asp = result.getDataDescription().getAspect();

      // Anzeige
      
      AnzeigeAktorTls anzeigeAktorUms = (AnzeigeAktorTls)m_listeAnzeigeAktorTlsDes.get(object.getPid());
      
      if (anzeigeAktorUms != null)
      {
        m_debug.fine("Umsetzung " + anzeigeAktorUms.getAnzeigeTypUmsetzung().getPid() + " " + result.toString());
  
        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktorUms.getAnzeigeTypUmsetzung();
        
        int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();
  
        for (int i = 0; i < anzahlGlobalUmsetzung; i++)
        {
          GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(i);
          
          if (globalUmsetzung.getUmsetzungsModul() != null)
          {
          	String klassenName = globalUmsetzung.getUmsetzungsModul();
          	
          	klassenName = klassenName.replace("zswzg.kex.", "de.bsvrz.ste.agzs.");
            
            if (anzeigeUmsetzungAbarbeiten(klassenName, result, object, atg, asp, anzeigeTypUmsetzungTls, i))
            {
              datenVerarbeitet = true;
            }
          }
        }
        
        if (!datenVerarbeitet) // Default Umsetzungs-Klasse
        {
          anzeigeUmsetzungAbarbeiten(klassenNameDefaultAnzeige, result, object, atg, asp, anzeigeTypUmsetzungTls, -1);
        }
      }

      // AnzeigeQuerschnitt
      
      AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktorUms = (AnzeigeQuerschnittAktorTls)m_listeAnzeigeQuerschnittAktorTlsDes.get(object.getPid());
      
      if (anzeigeQuerschnittAktorUms != null)
      {
        m_debug.fine("Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid() + " " + result.toString());
  
        AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();
        
        int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeGlobalUmsetzung().size();
  
        for (int i = 0; i < anzahlGlobalUmsetzung; i++)
        {
          GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeGlobalUmsetzung().get(i);
          
          if (globalUmsetzung.getUmsetzungsModul() != null)
          {
          	String klassenName = globalUmsetzung.getUmsetzungsModul();
          	
          	klassenName = klassenName.replace("zswzg.kex.", "de.bsvrz.ste.agzs.");
          	
            if (anzeigeQuerschnittUmsetzungAbarbeiten(klassenName, result, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, i))
            {
              datenVerarbeitet = true;
            }
          }
        }
        
        if (!datenVerarbeitet) // Default Umsetzungs-Klasse
        {
          anzeigeQuerschnittUmsetzungAbarbeiten(klassenNameDefaultAnzeigeQuerschnitt, result, object, atg, asp, anzeigeQuerschnittTypUmsetzungTls, -1);
        }
      }
      
      // Dirigent
      
//      System.out.println("Dirigent DE " + object.getPid());
      
      DirigentAktorTls dirigentAktorUms = (DirigentAktorTls)m_listeDirigentAktorTlsDes.get(object.getPid());

//      System.out.println("dirigentAktorUms " + dirigentAktorUms);
      
      if (dirigentAktorUms != null)
      {
        m_debug.fine("Umsetzung " + dirigentAktorUms.getDirigentUmsetzung().getPid() + " " + result.toString());
  
        DirigentUmsetzungTls dirigentUmsetzungTls = dirigentAktorUms.getDirigentUmsetzung();
        
        int anzahlGlobalUmsetzung = dirigentUmsetzungTls.getListeGlobalUmsetzung().size();
  
        for (int i = 0; i < anzahlGlobalUmsetzung; i++)
        {
          GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)dirigentUmsetzungTls.getListeGlobalUmsetzung().get(i);
          
          if (globalUmsetzung.getUmsetzungsModul() != null)
          {
          	String klassenName = globalUmsetzung.getUmsetzungsModul();
          	
          	klassenName = klassenName.replace("zswzg.kex.", "de.bsvrz.ste.agzs.");
          	
            if (dirigentUmsetzungAbarbeiten(klassenName, result, object, atg, asp, dirigentUmsetzungTls, i))
            {
              datenVerarbeitet = true;
            }
          }
        }
        
        if (!datenVerarbeitet) // Default Umsetzungs-Klasse
        {
          dirigentUmsetzungAbarbeiten(klassenNameDefaultDirigent, result, object, atg, asp, dirigentUmsetzungTls, -1);
        }
      }
    }
  }
  
  /**
   * Methode fr die Abarbeitung der Umsetzung fr Anzeigen.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param object Anzeige
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param anzeigeTypUmsetzungTls AnzeigeTypUmsetzungTls fr diese Anzeige.
   * @param indexUmsetzung Index der Umsetzung.
   * @return <code>true</code>, wenn Umsetzung durchgefhrt wurde, sonst <code>false</code>.
   */
  public boolean anzeigeUmsetzungAbarbeiten(String klassenName, ResultData result, SystemObject object, AttributeGroup atg, Aspect asp, AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int indexUmsetzung)
  {
    boolean verarbeitet = false;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeUmsetzungAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);
    
      Class[] formaleParameter = new Class[4];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_listeAnzeigeAktorTlsDes, m_listeAnzeigeAktorTlsAnzeige, m_listeAnzeigeEigenschaftIst, m_listeAnzeigeStatus});

      Method methode2 = null;
      
      String atgPublikation = null;
      String aspPublikation = null;
      
      for (int i = 0; i < m_anzeigeMethoden.length; i++)
      {
        if (atg.getPid().equals(m_anzeigeMethoden[i][0]))
        {
          if ((m_anzeigeMethoden[i][1].length() == 0) || (asp.getPid().equals(m_anzeigeMethoden[i][1])))
          {
            aktuelleMethode = m_anzeigeMethoden[i][2];

            atgPublikation = m_anzeigeMethoden[i][3];
            aspPublikation = m_anzeigeMethoden[i][4];

            methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class});

            break; // gefunden
          }
        }
      }
      
      Data erg = null;
      
      if (methode2 != null)
      {
        erg = (Data)methode2.invoke(ret, new Object[]{result});

        verarbeitet = true;
      }
      
      if (erg != null)
      {
        AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)m_listeAnzeigeAktorTlsDes.get(object.getPid());
        
        if (anzeigeAktor != null)
        {
          SystemObject objectSend = anzeigeAktor.getAnzeige();
        
          DataDescription dd_publizieren  = new DataDescription(m_dataModel.getAttributeGroup(atgPublikation), m_dataModel.getAspect(aspPublikation));
          ResultData resultSend = new ResultData(objectSend, dd_publizieren, result.getDataTime(), erg);

//          System.out.println("Publizieren " + objectSend.getPid());
          
          publizieren((ClientDavConnection)m_connection, objectSend, m_dataModel.getAttributeGroup(atgPublikation), m_dataModel.getAspect(aspPublikation), resultSend);
        }
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der AnzeigeUmsetzung konnte nicht geladen werden.");

      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");
     
      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");
      
      e.printStackTrace();
    }

    return verarbeitet;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung fr AnzeigeQuerschnitte.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param object Anzeige
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param anzeigeQuerschnittTypUmsetzungTls AnzeigeQuerschnittTypUmsetzungTls fr diesen AnzeigeQuerschnitt.
   * @param indexUmsetzung Index der Umsetzung.
   * @return <code>true</code>, wenn Umsetzung durchgefhrt wurde, sonst <code>false</code>.
   */
  public boolean anzeigeQuerschnittUmsetzungAbarbeiten(String klassenName, ResultData result, SystemObject object, AttributeGroup atg, Aspect asp, AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls, int indexUmsetzung)
  {
    boolean verarbeitet = false;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeQuerschnittUmsetzungAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);
    
      Class[] formaleParameter = new Class[5];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
      formaleParameter[4] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_listeAnzeigeQuerschnittAktorTlsDes, m_listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt, m_listeAnzeigeQuerschnittEigenschaftIst, m_listeAnzeigeQuerschnittSchaltbarkeitNba, m_listeAnzeigeQuerschnittStatus});

      Method methode2 = null;
      
      for (int i = 0; i < m_anzeigeQuerschnittMethoden.length; i++)
      {
        if (atg.getPid().equals(m_anzeigeQuerschnittMethoden[i][0]))
        {
          if ((m_anzeigeQuerschnittMethoden[i][1].length() == 0) || (asp.getPid().equals(m_anzeigeQuerschnittMethoden[i][1])))
          {
            aktuelleMethode = m_anzeigeQuerschnittMethoden[i][2];

            methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class});

            break; // gefunden
          }
        }
      }
      
      Data erg = null;
      
      if (methode2 != null)
      {
        erg = (Data)methode2.invoke(ret, new Object[]{result});

        verarbeitet = true;
      }
      
      if (erg != null)
      {
        AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)m_listeAnzeigeQuerschnittAktorTlsDes.get(object.getPid());
        
        if (anzeigeQuerschnittAktor != null)
        {
          SystemObject objectSend = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt();
          
          DataDescription dd_publizieren  = new DataDescription(m_dataModel.getAttributeGroup(erg.getName()), m_dataModel.getAspect("asp.aqZustand"));
          ResultData resultSend = new ResultData(objectSend, dd_publizieren, result.getDataTime(), erg);
  
          publizieren((ClientDavConnection)m_connection, objectSend, m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"), resultSend);
        }
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der AnzeigeQuerschnittUmsetzung konnte nicht geladen werden.");

      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }

    return verarbeitet;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung fr Dirigenten.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param object Anzeige
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param dirigentUmsetzungTls DirigentUmsetzungTls fr diesen Dirigenten.
   * @param indexUmsetzung Index der Umsetzung.
   * @return <code>true</code>, wenn Umsetzung durchgefhrt wurde, sonst <code>false</code>.
   */
  public boolean dirigentUmsetzungAbarbeiten(String klassenName, ResultData result, SystemObject object, AttributeGroup atg, Aspect asp, DirigentUmsetzungTls dirigentUmsetzungTls, int indexUmsetzung)
  {
    boolean verarbeitet = false;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("dirigentUmsetzungAbarbeiten " + klassenName + " object " + object + " result " + result);
    System.out.println("dirigentUmsetzungAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);
    
      Class[] formaleParameter = new Class[4];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = HashMap.class;
      formaleParameter[3] = HashMap.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_listeDirigentAktorTlsDes, m_listeDirigentAktorTlsDirigent, m_listeDirigentEigenschaftIst, m_listeDirigentStatus});

      Method methode2 = null;
      
      for (int i = 0; i < m_dirigentMethoden.length; i++)
      {
        if (atg.getPid().equals(m_dirigentMethoden[i][0]))
        {
          if ((m_dirigentMethoden[i][1].length() == 0) || (asp.getPid().equals(m_dirigentMethoden[i][1])))
          {
            aktuelleMethode = m_dirigentMethoden[i][2];
            
            methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class});

            break; // gefunden
          }
        }
      }
      
      Data erg = null;
      
      if (methode2 != null)
      {
        erg = (Data)methode2.invoke(ret, new Object[]{result});

        verarbeitet = true;
      }
      
      if (erg != null)
      {
        DirigentAktorTls dirigentAktor = (DirigentAktorTls)m_listeDirigentAktorTlsDes.get(object.getPid());
        
        if (dirigentAktor != null)
        {
          SystemObject objectSend = dirigentAktor.getDirigent();
        
          DataDescription dd_publizieren  = new DataDescription(m_dataModel.getAttributeGroup("atg.dirigentEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"));
          ResultData resultSend = new ResultData(objectSend, dd_publizieren, result.getDataTime(), erg);

//          System.out.println("Publizieren " + objectSend.getPid());
          
          publizieren((ClientDavConnection)m_connection, objectSend, m_dataModel.getAttributeGroup("atg.dirigentEigenschaftIst"), m_dataModel.getAspect("asp.aqZustand"), resultSend);
        }
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der DirigentUmsetzung konnte nicht geladen werden.");

      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");
     
      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");
      
      e.printStackTrace();
    }

    return verarbeitet;
  }

  /**
   * Holen der HashMap mit den AnzeigeAktorTls mit Key DEs.
   * 
   * @return HashMap mit den AnzeigeAktorTls mit Key DEs.
   */
  public HashMap getListeAnzeigeAktorTlsDes()
  {
    return m_listeAnzeigeAktorTlsDes;
  }

  /**
   * Holen der HashMap mit den AnzeigeQuerschnittAktorTls mit Key DEs.
   * 
   * @return HashMap mit den AnzeigeQuerschnittAktorTls mit Key DEs.
   */
  public HashMap getListeAnzeigeQuerschnittAktorTlsDes()
  {
    return m_listeAnzeigeQuerschnittAktorTlsDes;
  }
  
  /**
   * Holen der HashMap mit den DirigentAktorTls mit Key DEs.
   * 
   * @return HashMap mit den DirigentAktorTls mit Key DEs.
   */
  public HashMap getListeDirigentAktorTlsDes()
  {
    return m_listeDirigentAktorTlsDes;
  }
}
